@@ -29,17 +29,38 @@
 
 #define CACHE_SIZE		"/apps/gnome-mplayer/preferences/cache_size"
 #define OSDLEVEL		"/apps/gnome-mplayer/preferences/osdlevel"
+#define PPLEVEL		"/apps/gnome-mplayer/preferences/pplevel"
 #define SOFTVOL			"/apps/gnome-mplayer/preferences/softvol"
 #define VERBOSE			"/apps/gnome-mplayer/preferences/verbose"
 #define VERTICAL		"/apps/gnome-mplayer/preferences/vertical"
 #define FORCECACHE		"/apps/gnome-mplayer/preferences/forcecache"
 #define LAST_DIR		"/apps/gnome-mplayer/preferences/last_dir"
 #define SHOWPLAYLIST	"/apps/gnome-mplayer/preferences/showplaylist"
+#define DISABLEASS          "/apps/gnome-mplayer/preferences/disableass"
+#define DISABLEEMBEDDEDFONTS    "/apps/gnome-mplayer/preferences/disableembeddedfonts"
+#define DISABLEDEINTERLACE	"/apps/gnome-mplayer/preferences/disable_deinterlace"
+#define DISABLEFRAMEDROP	"/apps/gnome-mplayer/preferences/disableframedrop"
+#define DISABLEFULLSCREEN	"/apps/gnome-mplayer/preferences/disablefullscreen"
+#define DISABLECONTEXTMENU	"/apps/gnome-mplayer/preferences/disablecontextmenu"
+#define DISABLEPAUSEONCLICK	"/apps/gnome-mplayer/preferences/disable_pause_on_click"
+#define METADATACODEPAGE	"/apps/gnome-mplayer/preferences/metadatacodepage"
+#define SUBTITLEFONT	"/apps/gnome-mplayer/preferences/subtitlefont"
+#define SUBTITLESCALE	"/apps/gnome-mplayer/preferences/subtitlescale"
+#define SUBTITLECODEPAGE	"/apps/gnome-mplayer/preferences/subtitlecodepage"
+#define SUBTITLECOLOR	"/apps/gnome-mplayer/preferences/subtitlecolor"
+
+#define MPLAYER_BIN		"/apps/gnome-mplayer/preferences/mplayer_bin"
+#define EXTRAOPTS		"/apps/gnome-mplayer/preferences/extraopts"
 
 #define DISABLE_QT		"/apps/gecko-mediaplayer/preferences/disable_qt"
 #define DISABLE_REAL	"/apps/gecko-mediaplayer/preferences/disable_real"
 #define DISABLE_WMP		"/apps/gecko-mediaplayer/preferences/disable_wmp"
 #define DISABLE_DVX		"/apps/gecko-mediaplayer/preferences/disable_dvx"
+#define DISABLE_EMBEDDING		"/apps/gecko-mediaplayer/preferences/disable_embedding"
+
+#define REMEMBER_LOC		"/apps/gnome-mplayer/preferences/remember_loc"
+#define WINDOW_X		"/apps/gnome-mplayer/preferences/window_x"
+#define WINDOW_Y		"/apps/gnome-mplayer/preferences/window_y"
 
 
 // JavaScript Playstates
@@ -78,6 +99,7 @@ typedef struct _IdleData {
     gchar info[1024];
     gchar media_info[2048];
     gchar url[1024];
+    gchar *device;
     gdouble percent;
     gdouble cachepercent;
     gint streaming;
@@ -109,6 +131,7 @@ typedef struct _IdleData {
     gchar audio_codec[16];
     gchar audio_bitrate[16];
     gchar audio_samplerate[16];
+    gchar audio_channels[16];
     gchar metadata[1024];
     gboolean fromdbus;
 } IdleData;
@@ -122,7 +145,7 @@ typedef struct _ThreadData {
     gint streaming;
     gint player_window;
     gint playlist;
-	gboolean done;
+    gboolean done;
 } ThreadData;
 
 enum {
@@ -136,10 +159,19 @@ enum {
     N_COLUMNS
 };
 
+//Define MIME for DnD
+#define DRAG_NAME_0		"text/plain"
+#define DRAG_INFO_0		0
+#define DRAG_NAME_1		"text/uri-list"
+#define DRAG_INFO_1		1
+#define DRAG_NAME_2		"STRING"
+#define DRAG_INFO_2		2
+
 gchar *lastfile;
 gint cache_size;
 gboolean forcecache;
 gint osdlevel;
+gint pplevel;
 gint streaming;
 gint showcontrols;
 gint fullscreen;
@@ -149,8 +181,9 @@ gint embed_window;
 gint window_x;
 gint window_y;
 gint control_id;
-gint softvol;
+gboolean softvol;
 gint verbose;
+gint reallyverbose;
 gint autostart;
 gint actual_x, actual_y;
 gint play_x, play_y;
@@ -160,7 +193,11 @@ gint stored_window_width, stored_window_height;
 gchar vm[10];
 gchar *vo;
 gchar *ao;
-gint disable_context_menu;
+gboolean disable_deinterlace;
+gboolean disable_framedrop;
+gboolean disable_context_menu;
+gboolean disable_fullscreen;
+gboolean disable_pause_on_click;
 gint loop;
 gint random_order;
 gboolean dontplaynext;
@@ -177,9 +214,25 @@ gboolean ok_to_play;
 gchar *subtitle;
 gchar *alang;
 gchar *slang;
+gchar *metadata_codepage;
 gint volume;
 gboolean vertical_layout;
 gboolean playlist_visible;
+gboolean restore_playlist;
+gboolean disable_ass;
+gboolean disable_embeddedfonts;
+gchar *subtitlefont;
+gdouble subtitle_scale;
+gchar *subtitle_codepage;
+gchar *subtitle_color;
+gboolean quit_on_complete;
+gchar *mplayer_bin;
+gchar *extraopts;
+gboolean single_instance;
+
+gboolean remember_loc;
+gint loc_window_x;
+gint loc_window_y;
 
 // tv stuff
 gchar *tv_device;
@@ -190,11 +243,13 @@ gint tv_height;
 gint tv_fps;
 
 GThread *thread;
+GMutex *slide_away;
 
 gboolean qt_disabled;
 gboolean real_disabled;
 gboolean wmp_disabled;
 gboolean dvx_disabled;
+gboolean embedding_disabled;
 
 // playlist stuff
 GtkListStore *playliststore;
@@ -242,6 +297,7 @@ void dbus_send_rpsignal_with_double(gchar * signal, gdouble value);
 void dbus_send_rpsignal_with_string(gchar * signal, gchar * value);
 void dbus_send_event(gchar * event, gint button);
 void dbus_unhook();
+void dbus_enable_screensaver();
 void dbus_disable_screensaver();
 
 void shutdown();
