@@ -69,7 +69,7 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
     gint count;
     GtkTreePath *treepath;
     gint source_id;
-	gint bitrate;
+    gint bitrate;
 
     message_type = dbus_message_get_type(message);
     sender = dbus_message_get_sender(message);
@@ -84,7 +84,6 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
     path1 = g_strdup_printf("/control/%i", control_id);
     path2 = g_strdup_printf("/window/%i", embed_window);
     path3 = g_strdup_printf("/pid/%i", getpid());
-
     path4 = g_strdup_printf("/console/%s", rpconsole);
 
     if (dbus_message_get_path(message)) {
@@ -94,7 +93,9 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
             g_ascii_strcasecmp(dbus_message_get_path(message), path2) == 0 ||
             g_ascii_strcasecmp(dbus_message_get_path(message), path3) == 0 ||
             g_ascii_strcasecmp(dbus_message_get_path(message), path4) == 0 ||
-            g_ascii_strcasecmp(dbus_message_get_path(message), "/org/gnome/SettingsDaemon") == 0) {
+            g_ascii_strcasecmp(dbus_message_get_path(message), "/org/gnome/SettingsDaemon") == 0 ||
+            g_ascii_strcasecmp(dbus_message_get_path(message),
+                               "/org/gnome/SettingsDaemon/MediaKeys") == 0) {
 
             // printf("Path matched %s\n", dbus_message_get_path(message));
             if (message_type == DBUS_MESSAGE_TYPE_SIGNAL) {
@@ -132,8 +133,9 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                                         gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore),
                                                                 &iter);
                                     gtk_tree_selection_select_path(selection, treepath);
-									if (GTK_IS_WIDGET(list))
-										gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list),treepath,NULL,FALSE,0,0);
+                                    if (GTK_IS_WIDGET(list))
+                                        gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list), treepath,
+                                                                     NULL, FALSE, 0, 0);
                                     gtk_tree_path_free(treepath);
                                 }
                             }
@@ -169,8 +171,9 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                             treepath =
                                 gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &iter);
                             gtk_tree_selection_select_path(selection, treepath);
-							if (GTK_IS_WIDGET(list))
-								gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list),treepath,NULL,FALSE,0,0);
+                            if (GTK_IS_WIDGET(list))
+                                gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list), treepath, NULL,
+                                                             FALSE, 0, 0);
                             gtk_tree_path_free(treepath);
                         }
 
@@ -359,6 +362,24 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "VolumeUp") == 0
+                    && idledata != NULL) {
+                    if (idledata->volume < 100) {
+                        idledata->volume++;
+                        g_idle_add(set_volume, idledata);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "VolumeDown") == 0
+                    && idledata != NULL) {
+                    if (idledata->volume > 0) {
+                        idledata->volume--;
+                        g_idle_add(set_volume, idledata);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "RP_Volume") == 0
                     && idledata != NULL) {
                     dbus_error_init(&error);
@@ -753,13 +774,14 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
                 if (dbus_message_is_method_call(message, "com.gnome.mplayer", "GetBitrate")) {
-					dbus_error_init(&error);
-					if (dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {					
-						bitrate = get_bitrate(s);
-					} else {
-						dbus_error_free(&error);
-						bitrate = 0;
-					}
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
+                        bitrate = get_bitrate(s);
+                    } else {
+                        dbus_error_free(&error);
+                        bitrate = 0;
+                    }
                     reply_message = dbus_message_new_method_return(message);
                     dbus_message_append_args(reply_message, DBUS_TYPE_INT32, &bitrate,
                                              DBUS_TYPE_INVALID);
@@ -777,6 +799,7 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
     g_free(path1);
     g_free(path2);
     g_free(path3);
+    g_free(path4);
     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
@@ -986,6 +1009,7 @@ gboolean dbus_hookup(gint windowid, gint controlid)
     gchar *path;
     DBusMessage *reply_message;
     gint id;
+    gint ret;
 
     dbus_error_init(&error);
     connection = dbus_bus_get_private(type, &error);
@@ -1017,6 +1041,13 @@ gboolean dbus_hookup(gint windowid, gint controlid)
     g_free(match);
     dbus_error_free(&error);
 
+    match = g_strdup_printf("type='signal',interface='org.gnome.SettingsDaemon.MediaKeys'");
+    dbus_bus_add_match(connection, match, &error);
+    if (verbose)
+        printf("Using match: %s\n", match);
+    g_free(match);
+    dbus_error_free(&error);
+
     dbus_connection_add_filter(connection, filter_func, NULL, NULL);
 
     if (verbose)
@@ -1024,8 +1055,10 @@ gboolean dbus_hookup(gint windowid, gint controlid)
 
     if (control_id != 0) {
         path = g_strdup_printf("com.gnome.mplayer.cid%i", control_id);
-        dbus_bus_request_name(connection, path, 0, NULL);
+        ret = dbus_bus_request_name(connection, path, 0, NULL);
         g_free(path);
+        if (single_instance && ret > 1)
+            exit(1);
         path = g_strdup_printf("/control/%i", control_id);
         id = control_id;
         reply_message = dbus_message_new_signal(path, "com.gecko.mediaplayer", "Ready");
@@ -1034,7 +1067,9 @@ gboolean dbus_hookup(gint windowid, gint controlid)
         dbus_message_unref(reply_message);
         g_free(path);
     } else {
-        dbus_bus_request_name(connection, "com.gnome.mplayer", 0, NULL);
+        ret = dbus_bus_request_name(connection, "com.gnome.mplayer", 0, NULL);
+        if (single_instance && ret > 1)
+            exit(1);
     }
 
 
@@ -1059,6 +1094,20 @@ void dbus_unhook()
     }
 }
 
+void dbus_enable_screensaver()
+{
+    DBusMessage *message;
+
+    if (connection != NULL) {
+        message =
+            dbus_message_new_method_call("org.gnome.ScreenSaver", "/org/gnome/ScreenSaver",
+                                         "org.gnome.ScreenSaver", "UnInhibit");
+        dbus_message_append_args(message, DBUS_TYPE_INT32, &cookie, DBUS_TYPE_INVALID);
+        dbus_connection_send(connection, message, NULL);
+        dbus_message_unref(message);
+    }
+}
+
 void dbus_disable_screensaver()
 {
     DBusError error;
