@@ -21,9 +21,10 @@
  * 	51 Franklin Street, Fifth Floor
  * 	Boston, MA  02110-1301, USA.
  */
-
+//#include <X11/Xlib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
+//#include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib.h>
 #include <glib/gstdio.h>
@@ -33,22 +34,6 @@
 #include <gconf/gconf-value.h>
 #include "playlist.h"
 
-/*
-#include "../pixmaps/play_down_small.xpm"
-#include "../pixmaps/play_up_small.xpm"
-#include "../pixmaps/pause_down_small.xpm"
-#include "../pixmaps/pause_up_small.xpm"
-#include "../pixmaps/stop_down_small.xpm"
-#include "../pixmaps/stop_up_small.xpm"
-#include "../pixmaps/ff_down_small.xpm"
-#include "../pixmaps/ff_up_small.xpm"
-#include "../pixmaps/rew_down_small.xpm"
-#include "../pixmaps/rew_up_small.xpm"
-#include "../pixmaps/fs_down_small.xpm"
-#include "../pixmaps/fs_up_small.xpm"
-*/
-
-
 GtkWidget *window;
 GdkWindow *window_container;
 
@@ -57,9 +42,17 @@ GtkMenuItem *menuitem_file;
 GtkMenu *menu_file;
 GtkMenuItem *menuitem_file_open;
 GtkMenuItem *menuitem_file_open_location;
+GtkMenuItem *menuitem_file_dvd;
+GtkMenu *menu_file_dvd;
 GtkMenuItem *menuitem_file_open_dvd;
 GtkMenuItem *menuitem_file_open_dvdnav;
+GtkMenuItem *menuitem_file_open_dvd_folder;
+GtkMenuItem *menuitem_file_open_dvdnav_folder;
 GtkMenuItem *menuitem_file_open_acd;
+GtkMenuItem *menuitem_file_tv;
+GtkMenu *menu_file_tv;
+GtkMenuItem *menuitem_file_open_atv;
+GtkMenuItem *menuitem_file_open_dtv;
 GtkMenuItem *menuitem_file_open_playlist;
 GtkMenuItem *menuitem_file_sep1;
 GtkMenuItem *menuitem_file_details;
@@ -70,6 +63,8 @@ GtkMenu *menu_edit;
 GtkMenuItem *menuitem_edit_random;
 GtkMenuItem *menuitem_edit_loop;
 GtkMenuItem *menuitem_edit_switch_audio;
+GtkMenuItem *menuitem_edit_set_subtitle;
+GtkMenuItem *menuitem_edit_take_screenshot;
 GtkMenuItem *menuitem_edit_sep1;
 GtkMenuItem *menuitem_edit_config;
 GtkMenuItem *menuitem_help;
@@ -92,6 +87,7 @@ GtkMenuItem *menuitem_view_aspect_sixteen_nine;
 GtkMenuItem *menuitem_view_aspect_sixteen_ten;
 GtkMenuItem *menuitem_view_sep2;
 GtkMenuItem *menuitem_view_subtitles;
+GtkMenuItem *menuitem_view_angle;
 GtkMenuItem *menuitem_view_controls;
 GtkMenuItem *menuitem_view_sep3;
 GtkMenuItem *menuitem_view_advanced;
@@ -108,7 +104,6 @@ GtkMenuItem *menuitem_stop;
 GtkMenuItem *menuitem_sep1;
 GtkMenuItem *menuitem_copyurl;
 GtkMenuItem *menuitem_sep2;
-GtkMenuItem *menuitem_sep3;
 GtkMenuItem *menuitem_sep4;
 GtkMenuItem *menuitem_save;
 GtkMenuItem *menuitem_showcontrols;
@@ -125,6 +120,8 @@ GtkWidget *controls_box;
 
 GtkWidget *fixed;
 GtkWidget *media_label;
+GtkWidget *details_vbox;
+GtkWidget *details_table;
 
 GtkWidget *drawing_area;
 GdkPixbuf *pb_play;
@@ -135,6 +132,7 @@ GdkPixbuf *pb_rew;
 GdkPixbuf *pb_fs;
 GdkPixbuf *pb_next;
 GdkPixbuf *pb_prev;
+GdkPixbuf *pb_menu;
 GdkPixbuf *pb_icon;
 GdkPixbuf *pb_logo;
 GdkPixbuf *pb_button;
@@ -154,6 +152,7 @@ GtkWidget *ff_event_box;
 GtkWidget *rew_event_box;
 GtkWidget *prev_event_box;
 GtkWidget *next_event_box;
+GtkWidget *menu_event_box;
 
 GtkWidget *fs_event_box;
 GtkProgressBar *progress;
@@ -168,30 +167,47 @@ GtkWidget *image_ff;
 GtkWidget *image_rew;
 GtkWidget *image_next;
 GtkWidget *image_prev;
+GtkWidget *image_menu;
 GtkWidget *image_fs;
 GtkWidget *image_icon;
 GtkTooltips *tooltip;
 GtkTooltips *volume_tip;
-GtkWidget *song_title;
 
 GtkWidget *config_vo;
 GtkWidget *config_ao;
 GtkWidget *config_cachesize;
 GtkWidget *config_osdlevel;
+GtkWidget *config_deinterlace;
+GtkWidget *config_framedrop;
+GtkWidget *config_pplevel;
 
 GtkWidget *config_playlist_visible;
 GtkWidget *config_vertical_layout;
+GtkWidget *config_pause_on_click;
 GtkWidget *config_softvol;
 GtkWidget *config_forcecache;
 GtkWidget *config_verbose;
 
 GtkWidget *config_alang;
 GtkWidget *config_slang;
+GtkWidget *config_metadata_codepage;
+
+GtkWidget *config_ass;
+GtkWidget *config_embeddedfonts;
+GtkWidget *config_subtitle_font;
+GtkWidget *config_subtitle_scale;
+GtkWidget *config_subtitle_codepage;
+GtkWidget *config_subtitle_color;
 
 GtkWidget *config_qt;
 GtkWidget *config_real;
 GtkWidget *config_wmp;
 GtkWidget *config_dvx;
+GtkWidget *config_noembed;
+
+GtkWidget *config_mplayer_bin;
+GtkWidget *config_extraopts;
+GtkWidget *config_remember_loc;
 
 GtkWidget *open_location;
 
@@ -201,14 +217,6 @@ GtkWidget *plvbox;
 
 GtkAccelGroup *accel_group;
 
-//Define MIME for DnD
-#define DRAG_NAME_0		"text/plain"
-#define DRAG_INFO_0		0
-#define DRAG_NAME_1		"text/uri-list"
-#define DRAG_INFO_1		1
-#define DRAG_NAME_2		"STRING"
-#define DRAG_INFO_2		2
-
 glong last_movement_time;
 
 gboolean popup_handler(GtkWidget * widget, GdkEvent * event, void *data);
@@ -234,6 +242,7 @@ void menuitem_stop_callback(GtkMenuItem * menuitem, void *data);
 void menuitem_fs_callback(GtkMenuItem * menuitem, void *data);
 void menuitem_showcontrols_callback(GtkCheckMenuItem * menuitem, void *data);
 void menuitem_quit_callback(GtkMenuItem * menuitem, void *data);
+void menuitem_details_callback(GtkMenuItem * menuitem, void *data);
 
 gboolean playlist_drop_callback(GtkWidget * widget, GdkDragContext * dc,
                                 gint x, gint y, GtkSelectionData * selection_data,
