@@ -28,7 +28,7 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <mntent.h>
+#include <mntent_compat.h>
 #include <unistd.h>
 #include <string.h>
 #include <stdio.h>
@@ -56,7 +56,9 @@ static GOptionEntry entries[] = {
     {"height", 'h', 0, G_OPTION_ARG_INT, &window_y, N_("Height of window to embed in"), "Y"},
     {"controlid", 0, 0, G_OPTION_ARG_INT, &control_id, N_("Unique DBUS controller id"), "CID"},
     {"playlist", 0, 0, G_OPTION_ARG_NONE, &playlist, N_("File Argument is a playlist"), NULL},
-    {"verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose, N_("Show more ouput on the console"), NULL},
+    {"verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose, N_("Show more output on the console"), NULL},
+    {"reallyverbose", '\0', 0, G_OPTION_ARG_NONE, &reallyverbose,
+     N_("Show even more output on the console"), NULL},
     {"softvol", 0, 0, G_OPTION_ARG_NONE, &softvol, N_("Use mplayer software volume control"), NULL},
     {"volume", 0, 0, G_OPTION_ARG_INT, &volume, N_("Set initial volume percentage"), NULL},
     {"showcontrols", 0, 0, G_OPTION_ARG_INT, &showcontrols, N_("Show the controls in window"),
@@ -65,17 +67,33 @@ static GOptionEntry entries[] = {
      N_("Autostart the media default to 1, set to 0 to load but don't play"), "[0|1]"},
     {"disablecontextmenu", 0, 0, G_OPTION_ARG_NONE, &disable_context_menu,
      N_("Disable popup menu on right click"), NULL},
+    {"disablefullscreen", 0, 0, G_OPTION_ARG_NONE, &disable_fullscreen,
+     N_("Disable fullscreen options in browser mode"), NULL},
     {"loop", 0, 0, G_OPTION_ARG_NONE, &loop, N_("Play all files on the playlist forever"), NULL},
+    {"quit_on_complete", 'q', 0, G_OPTION_ARG_NONE, &quit_on_complete,
+     N_("Quit application when last file on playlist is played"), NULL},
     {"random", 0, 0, G_OPTION_ARG_NONE, &random_order, N_("Play items on playlist in random order"),
      NULL},
     {"cache", 0, 0, G_OPTION_ARG_INT, &cache_size, N_("Set cache size"),
      NULL},
     {"forcecache", 0, 0, G_OPTION_ARG_NONE, &forcecache, N_("Force cache usage on streaming sites"),
      NULL},
-	{"vertical", 0, 0, G_OPTION_ARG_NONE, &vertical_layout, N_("Use Vertical Layout"),
-     NULL},	
-	{"showplaylist", 0, 0, G_OPTION_ARG_NONE, &playlist_visible, N_("Start with playlist open"),
-     NULL},	
+    {"disabledeinterlace", 0, 0, G_OPTION_ARG_NONE, &disable_deinterlace,
+     N_("Disable the deinterlace filter"),
+     NULL},
+    {"disableframedrop", 0, 0, G_OPTION_ARG_NONE, &disable_framedrop,
+     N_("Don't skip drawing frames to better keep sync"),
+     NULL},
+    {"disableass", 0, 0, G_OPTION_ARG_NONE, &disable_ass,
+     N_("Use the old subtitle rendering system"),
+     NULL},
+    {"disableembeddedfonts", 0, 0, G_OPTION_ARG_NONE, &disable_embeddedfonts,
+     N_("Don't use fonts embedded on matroska files"),
+     NULL},
+    {"vertical", 0, 0, G_OPTION_ARG_NONE, &vertical_layout, N_("Use Vertical Layout"),
+     NULL},
+    {"showplaylist", 0, 0, G_OPTION_ARG_NONE, &playlist_visible, N_("Start with playlist open"),
+     NULL},
     {"rpname", 0, 0, G_OPTION_ARG_STRING, &rpname, N_("Real Player Name"), "NAME"},
     {"rpconsole", 0, 0, G_OPTION_ARG_STRING, &rpconsole, N_("Real Player Console ID"), "CONSOLE"},
     {"rpcontrols", 0, 0, G_OPTION_ARG_STRING, &rpcontrols, N_("Real Player Console Controls"),
@@ -88,6 +106,8 @@ static GOptionEntry entries[] = {
     {"tvwidth", 0, 0, G_OPTION_ARG_INT, &tv_width, N_("Width of TV input"), "WIDTH"},
     {"tvheight", 0, 0, G_OPTION_ARG_INT, &tv_height, N_("Height of TV input"), "HEIGHT"},
     {"tvfps", 0, 0, G_OPTION_ARG_INT, &tv_fps, N_("Frames per second from TV input"), "FPS"},
+    {"single_instance", 0, 0, G_OPTION_ARG_NONE, &single_instance, N_("Only allow one instance"),
+     "FPS"},
     {NULL}
 };
 
@@ -100,19 +120,28 @@ gint play_file(gchar * filename, gint playlist)
     GtkWidget *dialog;
     gchar *error_msg = NULL;
     gchar *subtitle = NULL;
+    GtkTreePath *path;
 
     if (verbose)
         printf("playing - %s\n", filename);
 
     shutdown();
     g_strlcpy(thread_data->filename, filename, 1024);
-	thread_data->done = FALSE;
-	
-    if (gtk_list_store_iter_is_valid(playliststore, &iter))
-        gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, SUBTITLE_COLUMN, &subtitle, -1);
+    thread_data->done = FALSE;
 
+    if (gtk_list_store_iter_is_valid(playliststore, &iter)) {
+        gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, SUBTITLE_COLUMN, &subtitle, -1);
+        if (GTK_IS_TREE_SELECTION(selection)) {
+            path = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &iter);
+            gtk_tree_selection_select_path(selection, path);
+            if (GTK_IS_WIDGET(list))
+                gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list), path, NULL, FALSE, 0, 0);
+            gtk_tree_path_free(path);
+        }
+    }
     if (subtitle != NULL) {
         g_strlcpy(thread_data->subtitle, subtitle, 1024);
+        g_free(subtitle);
     }
 
     if (g_ascii_strcasecmp(filename, "") != 0) {
@@ -154,6 +183,22 @@ gint play_file(gchar * filename, gint playlist)
         thread_data->streaming = streaming_media(thread_data->filename);
         idledata->streaming = thread_data->streaming;
         streaming = thread_data->streaming;
+        g_strlcpy(idledata->video_format, "", 64);
+        g_strlcpy(idledata->video_codec, "", 16);
+        g_strlcpy(idledata->video_fps, "", 16);
+        g_strlcpy(idledata->video_bitrate, "", 16);
+        g_strlcpy(idledata->audio_codec, "", 16);
+        g_strlcpy(idledata->audio_bitrate, "", 16);
+        g_strlcpy(idledata->audio_samplerate, "", 16);
+        g_strlcpy(idledata->audio_channels, "", 16);
+
+        // these next 3 lines are here to make sure the window is available for mplayer to draw to
+        // for some vo's (like xv) if the window is not visible and big enough the vo setup fails
+        gtk_widget_set_size_request(drawing_area, 16, 16);
+        gtk_widget_show_all(fixed);
+        gtk_widget_hide(menu_event_box);
+        while (gtk_events_pending())
+            gtk_main_iteration();
 
         if (autostart) {
             g_idle_add(hide_buttons, idledata);
@@ -166,8 +211,6 @@ gint play_file(gchar * filename, gint playlist)
     return 0;
 }
 
-
-
 int main(int argc, char *argv[])
 {
     struct stat buf;
@@ -195,8 +238,8 @@ int main(int argc, char *argv[])
     last_window_height = 0;
     showcontrols = 1;
     autostart = 1;
-    videopresent = 1;
-    disable_context_menu = 0;
+    videopresent = 0;
+    disable_context_menu = FALSE;
     dontplaynext = FALSE;
     idledata = (IdleData *) g_new0(IdleData, 1);
     idledata->videopresent = FALSE;
@@ -207,6 +250,14 @@ int main(int argc, char *argv[])
     idledata->gamma = 0;
     idledata->hue = 0;
     idledata->saturation = 0;
+    idledata->device = NULL;
+    g_strlcpy(idledata->video_format, "", 64);
+    g_strlcpy(idledata->video_codec, "", 64);
+    g_strlcpy(idledata->video_fps, "", 64);
+    g_strlcpy(idledata->video_bitrate, "", 64);
+    g_strlcpy(idledata->audio_codec, "", 64);
+    g_strlcpy(idledata->audio_bitrate, "", 64);
+    g_strlcpy(idledata->audio_samplerate, "", 64);
     selection = NULL;
     lastfile = NULL;
     path = NULL;
@@ -225,61 +276,119 @@ int main(int argc, char *argv[])
     softvol = 0;
     alang = NULL;
     slang = NULL;
+    metadata_codepage = NULL;
     playlistname = NULL;
     window_width = -1;
     window_height = -1;
     stored_window_width = -1;
     stored_window_height = -1;
-	cache_size = 0;
-	forcecache = FALSE;
-	volume = 0;
-	vertical_layout = FALSE;
-	playlist_visible = FALSE;
+    cache_size = 0;
+    forcecache = FALSE;
+    volume = 0;
+    vertical_layout = FALSE;
+    playlist_visible = FALSE;
+    disable_fullscreen = FALSE;
+    disable_framedrop = FALSE;
+    softvol = FALSE;
+    subtitlefont = NULL;
+    subtitle_codepage = NULL;
+    subtitle_color = NULL;
+    quit_on_complete = FALSE;
+    slide_away = NULL;
+    verbose = 0;
+    reallyverbose = 0;
+    embedding_disabled = FALSE;
+    disable_pause_on_click = FALSE;
+    mplayer_bin = NULL;
+    single_instance = FALSE;
 
     // call g_type_init or otherwise we can crash
     g_type_init();
     gconf = gconf_client_get_default();
     osdlevel = gconf_client_get_int(gconf, OSDLEVEL, NULL);
-    softvol = gconf_client_get_int(gconf, SOFTVOL, NULL);
+    pplevel = gconf_client_get_int(gconf, PPLEVEL, NULL);
+    softvol = gconf_client_get_bool(gconf, SOFTVOL, NULL);
     forcecache = gconf_client_get_bool(gconf, FORCECACHE, NULL);
     vertical_layout = gconf_client_get_bool(gconf, VERTICAL, NULL);
     playlist_visible = gconf_client_get_bool(gconf, SHOWPLAYLIST, NULL);
+    disable_deinterlace = gconf_client_get_bool(gconf, DISABLEDEINTERLACE, NULL);
+    disable_framedrop = gconf_client_get_bool(gconf, DISABLEFRAMEDROP, NULL);
+    disable_fullscreen = gconf_client_get_bool(gconf, DISABLEFULLSCREEN, NULL);
+    disable_context_menu = gconf_client_get_bool(gconf, DISABLECONTEXTMENU, NULL);
+    disable_ass = gconf_client_get_bool(gconf, DISABLEASS, NULL);
+    disable_embeddedfonts = gconf_client_get_bool(gconf, DISABLEEMBEDDEDFONTS, NULL);
+    disable_pause_on_click = gconf_client_get_bool(gconf, DISABLEPAUSEONCLICK, NULL);
+    metadata_codepage = gconf_client_get_string(gconf, METADATACODEPAGE, NULL);
+    subtitlefont = gconf_client_get_string(gconf, SUBTITLEFONT, NULL);
+    subtitle_scale = gconf_client_get_float(gconf, SUBTITLESCALE, NULL);
+    if (subtitle_scale < 0.25) {
+        subtitle_scale = 1.0;
+    }
+    subtitle_codepage = gconf_client_get_string(gconf, SUBTITLECODEPAGE, NULL);
+    subtitle_color = gconf_client_get_string(gconf, SUBTITLECOLOR, NULL);
+
     qt_disabled = gconf_client_get_bool(gconf, DISABLE_QT, NULL);
     real_disabled = gconf_client_get_bool(gconf, DISABLE_REAL, NULL);
     wmp_disabled = gconf_client_get_bool(gconf, DISABLE_WMP, NULL);
     dvx_disabled = gconf_client_get_bool(gconf, DISABLE_DVX, NULL);
+    embedding_disabled = gconf_client_get_bool(gconf, DISABLE_EMBEDDING, NULL);
 
-    context = g_option_context_new(_("- GNOME Media player based on MPlayer"));
+    mplayer_bin = gconf_client_get_string(gconf, MPLAYER_BIN, NULL);
+    if (!g_file_test(mplayer_bin, G_FILE_TEST_EXISTS)) {
+        g_free(mplayer_bin);
+        mplayer_bin = NULL;
+    }
+    extraopts = gconf_client_get_string(gconf, EXTRAOPTS, NULL);
+
+    remember_loc = gconf_client_get_bool(gconf, REMEMBER_LOC, NULL);
+    loc_window_x = gconf_client_get_int(gconf, WINDOW_X, NULL);
+    loc_window_y = gconf_client_get_int(gconf, WINDOW_Y, NULL);
+
+    context = g_option_context_new(_("[FILES...] - GNOME Media player based on MPlayer"));
+#ifdef GTK2_12_ENABLED
+    g_option_context_set_translation_domain(context, "UTF-8");
+    g_option_context_set_translate_func(context, (GTranslateFunc) gettext, NULL, NULL);
+#endif
     g_option_context_add_main_entries(context, entries, GETTEXT_PACKAGE);
     g_option_context_add_group(context, gtk_get_option_group(TRUE));
     g_option_context_parse(context, &argc, &argv, &error);
 
-	if (cache_size == 0)
-		cache_size = gconf_client_get_int(gconf, CACHE_SIZE, NULL);
+    if (verbose == 0)
+        verbose = gconf_client_get_int(gconf, VERBOSE, NULL);
+
+    if (reallyverbose)
+        verbose = 2;
+
+    if (verbose)
+        printf(_("GNOME MPlayer v%s\n"), VERSION);
+
+    if (cache_size == 0)
+        cache_size = gconf_client_get_int(gconf, CACHE_SIZE, NULL);
     if (cache_size == 0)
         cache_size = 2000;
     g_object_unref(G_OBJECT(gconf));
-	
-	if (volume > 0 && volume <= 100) {
-		idledata->volume = (gdouble)volume;
-	}
 
-	if (error != NULL) {
+    if (volume == 0) {
+        volume = (gint) get_alsa_volume();
+    }
+
+    if (volume > 0 && volume <= 100) {
+        idledata->volume = (gdouble) volume;
+    }
+
+    if (playlist_visible && control_id != 0)
+        playlist_visible = FALSE;
+
+    if (error != NULL) {
         printf("%s\n", error->message);
         printf(_
                ("Run 'gnome-mplayer --help' to see a full list of available command line options.\n"));
         return 1;
     }
 
-    if (verbose == 0)
-        verbose = gconf_client_get_int(gconf, VERBOSE, NULL);
-
     if (!g_thread_supported())
         g_thread_init(NULL);
 
-    if (verbose)
-        printf(_("GNOME MPlayer v%s\n"), VERSION);
-
     if (rpconsole == NULL)
         rpconsole = g_strdup("NONE");
 
@@ -303,6 +412,7 @@ int main(int argc, char *argv[])
 
     read_mplayer_config();
     thread_running = g_mutex_new();
+    slide_away = g_mutex_new();
 
     if (argv[fileindex] != NULL) {
         g_stat(argv[fileindex], &buf);
@@ -395,9 +505,9 @@ int main(int argc, char *argv[])
             }
         }
     } else {
-		if (playlist_visible == TRUE && embed_window == 0)
-			menuitem_view_playlist_callback(NULL,NULL);
-	}		
+        if (playlist_visible == TRUE && embed_window == 0)
+            menuitem_view_playlist_callback(NULL, NULL);
+    }
 
     dbus_hookup(embed_window, control_id);
 
