@@ -46,6 +46,7 @@ gboolean playlist_popup_handler(GtkWidget * widget, GdkEvent * event, void *data
     GtkMenu *menu;
     GdkEventButton *event_button;
 
+
     menu = GTK_MENU(widget);
     gtk_widget_grab_focus(fixed);
     if (event->type == GDK_BUTTON_PRESS) {
@@ -56,10 +57,47 @@ gboolean playlist_popup_handler(GtkWidget * widget, GdkEvent * event, void *data
             gtk_menu_popup(menu, NULL, NULL, NULL, NULL, event_button->button, event_button->time);
             return TRUE;
         }
+
     }
     return FALSE;
 }
 
+gboolean button_release_callback(GtkWidget * widget, GdkEvent * event, gpointer data)
+{
+    GtkTreePath *path;
+    // GtkTreeSelection *sel;
+    gint pos;
+    GdkEventButton *event_button;
+
+    event_button = (GdkEventButton *) event;
+
+    gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(list), event_button->x, event_button->y, &path,
+                                  NULL, NULL, NULL);
+    if (path != NULL) {
+        pos = (gint) g_strtod(gtk_tree_path_to_string(path), NULL);
+        gtk_tree_path_free(path);
+
+        if (pos == 0) {
+            gtk_widget_set_sensitive(up, FALSE);
+        } else {
+            gtk_widget_set_sensitive(up, TRUE);
+        }
+
+        if (pos + 1 == gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL)) {
+            gtk_widget_set_sensitive(down, FALSE);
+        } else {
+            gtk_widget_set_sensitive(down, TRUE);
+        }
+    } else {
+        //sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
+        //gtk_tree_selection_unselect_all(sel);
+        //gtk_widget_set_sensitive(up, FALSE);
+        //gtk_widget_set_sensitive(down, FALSE);
+    }
+
+    return FALSE;
+}
+
 void playlist_set_subtitle_callback(GtkMenuItem * menuitem, void *data)
 {
     GtkTreeSelection *sel;
@@ -67,19 +105,29 @@ void playlist_set_subtitle_callback(GtkMenuItem * menuitem, void *data)
     GtkTreeIter localiter;
     gchar *subtitle;
     GtkWidget *dialog;
+    gchar *path;
+    gchar *item;
+    gchar *p;
 
     sel = gtk_tree_view_get_selection(view);
 
     if (gtk_tree_selection_get_selected(sel, NULL, &localiter)) {
+        gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &localiter, ITEM_COLUMN, &item, -1);
+        path = g_strdup(item);
+        p = g_strrstr(path, "/");
+        if (p != NULL)
+            p[1] = '\0';
 
         dialog = gtk_file_chooser_dialog_new(_("Set Subtitle"),
                                              GTK_WINDOW(window),
                                              GTK_FILE_CHOOSER_ACTION_OPEN,
                                              GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                              GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
+        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), path);
         if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
             subtitle = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
             gtk_list_store_set(playliststore, &localiter, SUBTITLE_COLUMN, subtitle, -1);
+            g_free(subtitle);
         }
         gtk_widget_destroy(dialog);
     }
@@ -107,22 +155,27 @@ gboolean playlist_drop_callback(GtkWidget * widget, GdkDragContext * dc,
 
     if ((info == DRAG_INFO_0) || (info == DRAG_INFO_1) || (info == DRAG_INFO_2)) {
         filename = g_filename_from_uri((const gchar *) selection_data->data, NULL, NULL);
+        if (filename == NULL)
+            return FALSE;
         list = g_strsplit(filename, "\n", 0);
 
         while (list[i] != NULL) {
             g_strchomp(list[i]);
             if (strlen(list[i]) > 0) {
-printf("filename = '%s'\n",list[i]);
-                playlist = detect_playlist(list[i]);
-
-                if (!playlist) {
-                    add_item_to_playlist(list[i], playlist);
+                printf("filename = '%s'\n", list[i]);
+                if (g_file_test(list[i], G_FILE_TEST_IS_DIR)) {
+                    add_folder_to_playlist_callback(list[i], NULL);
                 } else {
-                    if (!parse_playlist(list[i])) {
-                        localiter = add_item_to_playlist(list[i], playlist);
+                    playlist = detect_playlist(list[i]);
+
+                    if (!playlist) {
+                        add_item_to_playlist(list[i], playlist);
+                    } else {
+                        if (!parse_playlist(list[i])) {
+                            localiter = add_item_to_playlist(list[i], playlist);
+                        }
                     }
                 }
-
             }
             i++;
         }
@@ -164,9 +217,13 @@ gboolean playlist_motion_callback(GtkWidget * widget, GdkEventMotion * event, gp
                 tip = g_strdup_printf("%s", iterfilename);
             } else {
                 tip = g_strdup_printf(_("Filename: %s\nSubtitle: %s"), iterfilename, itersubtitle);
+                g_free(itersubtitle);
             }
             gtk_tooltips_set_tip(playlisttip, widget, tip, NULL);
+            g_free(tip);
+            g_free(iterfilename);
         }
+        gtk_tree_path_free(localpath);
     }
     return FALSE;
 
@@ -295,7 +352,7 @@ void add_item_to_playlist_callback(gpointer data, gpointer user_data)
             add_item_to_playlist(filename, playlist);
         }
     }
-
+    g_free(filename);
 }
 
 gint compar(gpointer a, gpointer b)
@@ -308,7 +365,7 @@ void add_folder_to_playlist_callback(gpointer data, gpointer user_data)
     gchar *filename = (gchar *) data;
     GDir *dir;
     gchar *name;
-    gchar *subdir;
+    gchar *subdir = NULL;
     GSList *list = NULL;
 
     dir = g_dir_open(filename, 0, NULL);
@@ -319,6 +376,7 @@ void add_folder_to_playlist_callback(gpointer data, gpointer user_data)
                 subdir = g_strdup_printf("%s/%s", filename, name);
                 if (g_file_test(subdir, G_FILE_TEST_IS_DIR)) {
                     add_folder_to_playlist_callback((gpointer) subdir, NULL);
+                    g_free(subdir);
                 } else {
                     list = g_slist_prepend(list, subdir);
                     filecount++;
@@ -345,8 +403,8 @@ void add_to_playlist(GtkWidget * widget, void *data)
     gchar *last_dir;
     GtkTreeViewColumn *column;
     gchar *coltitle;
-	gint count;
-	
+    gint count;
+
     dialog = gtk_file_chooser_dialog_new(_("Open File"),
                                          GTK_WINDOW(window),
                                          GTK_FILE_CHOOSER_ACTION_OPEN,
@@ -377,11 +435,11 @@ void add_to_playlist(GtkWidget * widget, void *data)
     update_gui();
     g_object_unref(G_OBJECT(gconf));
     column = gtk_tree_view_get_column(GTK_TREE_VIEW(list), 0);
-	count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL);
-    coltitle = g_strdup_printf(ngettext("Item to Play","Items to Play",count));
+    count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL);
+    coltitle = g_strdup_printf(ngettext("Item to Play", "Items to Play", count));
     gtk_tree_view_column_set_title(column, coltitle);
     g_free(coltitle);
-	
+
     gtk_widget_destroy(dialog);
 }
 
@@ -425,7 +483,9 @@ void add_folder_to_playlist(GtkWidget * widget, void *data)
     update_gui();
     g_object_unref(G_OBJECT(gconf));
     gtk_widget_destroy(dialog);
-    message = g_markup_printf_escaped(ngettext("\n\tFound %i file\n","\n\tFound %i files\n",filecount), filecount);
+    message =
+        g_markup_printf_escaped(ngettext("\n\tFound %i file\n", "\n\tFound %i files\n", filecount),
+                                filecount);
     g_strlcpy(idledata->media_info, message, 1024);
     g_free(message);
     g_idle_add(set_media_label, idledata);
@@ -452,8 +512,8 @@ void remove_from_playlist(GtkWidget * widget, gpointer data)
             if (GTK_IS_TREE_SELECTION(selection)) {
                 path = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &iter);
                 gtk_tree_selection_select_path(selection, path);
-				if (GTK_IS_WIDGET(list))
-					gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list),path,NULL,FALSE,0,0);
+                if (GTK_IS_WIDGET(list))
+                    gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list), path, NULL, FALSE, 0, 0);
                 gtk_tree_path_free(path);
             }
         }
@@ -469,39 +529,114 @@ void clear_playlist(GtkWidget * widget, void *data)
     shutdown();
     gtk_list_store_clear(playliststore);
     gtk_list_store_clear(nonrandomplayliststore);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem_edit_random),FALSE);
+    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem_edit_random), FALSE);
 }
 
+void move_item_up(GtkWidget * widget, void *data)
+{
+    GtkTreeSelection *sel;
+    GtkTreeIter localiter, a, b;
+    GtkTreePath *path;
+    gint pos = 0;
+
+    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
+    if (gtk_tree_selection_get_selected(sel, NULL, &localiter)) {
+        path = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &localiter);
+        if (path != NULL) {
+            pos = (gint) g_strtod(gtk_tree_path_to_string(path), NULL);
+            gtk_tree_path_free(path);
+
+            if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(playliststore), &a, NULL, pos)) {
+                if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(playliststore), &b, NULL, pos - 1)) {
+                    gtk_list_store_swap(playliststore, &a, &b);
+                }
+            }
+            if (gtk_tree_model_iter_nth_child
+                (GTK_TREE_MODEL(nonrandomplayliststore), &a, NULL, pos)) {
+                if (gtk_tree_model_iter_nth_child
+                    (GTK_TREE_MODEL(nonrandomplayliststore), &b, NULL, pos - 1)) {
+                    gtk_list_store_swap(nonrandomplayliststore, &a, &b);
+                }
+            }
+        }
+        if (pos - 1 <= 0) {
+            gtk_widget_set_sensitive(up, FALSE);
+        } else {
+            gtk_widget_set_sensitive(up, TRUE);
+        }
+        gtk_widget_set_sensitive(down, TRUE);
+    }
+}
+
+void move_item_down(GtkWidget * widget, void *data)
+{
+    GtkTreeSelection *sel;
+    GtkTreeIter localiter, a, b;
+    GtkTreePath *path;
+    gint pos = 0;
+
+    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
+    if (gtk_tree_selection_get_selected(sel, NULL, &localiter)) {
+        path = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &localiter);
+        if (path != NULL) {
+            pos = (gint) g_strtod(gtk_tree_path_to_string(path), NULL);
+            gtk_tree_path_free(path);
+            if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(playliststore), &a, NULL, pos)) {
+                if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(playliststore), &b, NULL, pos + 1)) {
+                    gtk_list_store_swap(playliststore, &a, &b);
+                }
+            }
+            if (gtk_tree_model_iter_nth_child
+                (GTK_TREE_MODEL(nonrandomplayliststore), &a, NULL, pos)) {
+                if (gtk_tree_model_iter_nth_child
+                    (GTK_TREE_MODEL(nonrandomplayliststore), &b, NULL, pos + 1)) {
+                    gtk_list_store_swap(nonrandomplayliststore, &a, &b);
+                }
+            }
+        }
+        if (pos + 2 == gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL)) {
+            gtk_widget_set_sensitive(down, FALSE);
+        } else {
+            gtk_widget_set_sensitive(down, TRUE);
+        }
+        gtk_widget_set_sensitive(up, TRUE);
+    }
+
+}
 
 gboolean playlist_select_callback(GtkTreeView * view, GtkTreePath * path,
                                   GtkTreeViewColumn * column, gpointer data)
 {
 
-    gchar *filename;
+    gchar *filename = NULL;
     gint count;
     gint playlist;
+    gchar *cmd = NULL;
 
     if (gtk_tree_model_get_iter(GTK_TREE_MODEL(playliststore), &iter, path)) {
         gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN, &filename,
                            COUNT_COLUMN, &count, PLAYLIST_COLUMN, &playlist, -1);
-        dontplaynext = TRUE;
-        shutdown();
+
+        if (state == QUIT) {
+            play_file(filename, playlist);
+        } else {
+            cmd = g_strdup_printf("loadfile \"%s\"\n", filename);
+            send_command(cmd);
+            g_free(cmd);
+            if (state != PLAYING)
+                play_callback(NULL, NULL, NULL);
+        }
         set_media_info_name(filename);
-        play_file(filename, playlist);
         gtk_list_store_set(playliststore, &iter, COUNT_COLUMN, count + 1, -1);
         g_free(filename);
-        if (state == 3) {
-            dontplaynext = FALSE;
-        } else {
-            dontplaynext = TRUE;
-        }
+
     }
     return FALSE;
 }
 
 void playlist_close(GtkWidget * widget, void *data)
 {
-	menuitem_view_playlist_callback(NULL, NULL);	
+    menuitem_view_playlist_callback(NULL, NULL);
 }
 
 void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
@@ -525,14 +660,12 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
     GtkAccelGroup *accel_group;
     GValue value = { 0, };
     GtkTooltips *tooltip;
-    //GtkRequisition plreq;
     gchar *coltitle;
     gint x, y, depth;
-	gint count;
+    gint count;
+    GtkTargetEntry target_entry[3];
+    gint i = 0;
 
-    //if (GTK_IS_TREE_SELECTION(selection)){
-    //      return;
-    //}
     g_value_init(&value, G_TYPE_BOOLEAN);
 
     if (GTK_IS_WIDGET(plvbox)) {
@@ -540,23 +673,26 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
             if (idledata->videopresent == FALSE) {
                 gdk_window_get_geometry(window->window, &x, &y, &window_width, &window_height,
                                         &depth);
-				stored_window_width = -1;
-				stored_window_height = -1;
+                stored_window_width = -1;
+                stored_window_height = -1;
                 gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
                 gtk_widget_hide(GTK_WIDGET(fixed));
                 gtk_widget_show_all(media_label);
-				gtk_widget_show(vbox);				
+                gtk_widget_show(vbox);
                 gtk_widget_set_size_request(window, -1, -1);
             } else {
                 gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
                 gtk_widget_set_size_request(window, -1, -1);
                 gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, TRUE);
-				window_width = -1;
-				window_height = -1;
-				if (stored_window_width != -1)
-	            	gtk_window_resize(GTK_WINDOW(window), stored_window_width, stored_window_height);
+                window_width = -1;
+                window_height = -1;
+                if (stored_window_width != -1)
+                    gtk_window_resize(GTK_WINDOW(window), stored_window_width,
+                                      stored_window_height);
             }
             gtk_container_remove(GTK_CONTAINER(pane), plvbox);
+            stored_window_width = -1;
+            stored_window_height = -1;
             plvbox = NULL;
             selection = NULL;
         }
@@ -564,13 +700,16 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
     } else {
         gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
         if (idledata->videopresent == FALSE) {
-			if (window_width != -1)
-	            gtk_window_resize(GTK_WINDOW(window), window_width, window_height);
-			gtk_widget_hide(vbox);
+            if (window_width != -1)
+                gtk_window_resize(GTK_WINDOW(window), window_width, window_height);
+            gtk_widget_hide(vbox);
         } else {
-			gdk_window_get_geometry(window->window, &x, &y, &stored_window_width, &stored_window_height, &depth);		
-		}
-		plvbox = gtk_vbox_new(FALSE, 12);
+            if (!restore_playlist) {
+                gdk_window_get_geometry(window->window, &x, &y, &stored_window_width,
+                                        &stored_window_height, &depth);
+            }
+        }
+        plvbox = gtk_vbox_new(FALSE, 12);
         hbox = gtk_hbox_new(FALSE, 12);
         gtk_box_set_homogeneous(GTK_BOX(hbox), FALSE);
         box = gtk_hbox_new(FALSE, 10);
@@ -578,6 +717,7 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
         closebox = gtk_hbox_new(FALSE, 0);
 
         list = gtk_tree_view_new_with_model(GTK_TREE_MODEL(playliststore));
+        gtk_tree_view_set_reorderable(GTK_TREE_VIEW(list), TRUE);
         gtk_widget_add_events(list, GDK_BUTTON_PRESS_MASK);
         gtk_widget_set_size_request(GTK_WIDGET(list), -1, -1);
 
@@ -585,29 +725,52 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
         if (gtk_list_store_iter_is_valid(playliststore, &iter)) {
             path = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &iter);
             gtk_tree_selection_select_path(selection, path);
-			gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list),path,NULL,FALSE,0,0);
+            gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list), path, NULL, FALSE, 0, 0);
             gtk_tree_path_free(path);
         }
 
         g_signal_connect(G_OBJECT(list), "row_activated", G_CALLBACK(playlist_select_callback),
                          NULL);
+        g_signal_connect(G_OBJECT(list), "button-release-event",
+                         G_CALLBACK(button_release_callback), NULL);
         g_signal_connect(G_OBJECT(list), "motion-notify-event",
                          G_CALLBACK(playlist_motion_callback), NULL);
+        // Give the window the property to accept DnD
+        target_entry[i].target = DRAG_NAME_0;
+        target_entry[i].flags = 0;
+        target_entry[i++].info = DRAG_INFO_0;
+        target_entry[i].target = DRAG_NAME_1;
+        target_entry[i].flags = 0;
+        target_entry[i++].info = DRAG_INFO_1;
+        target_entry[i].target = DRAG_NAME_2;
+        target_entry[i].flags = 0;
+        target_entry[i++].info = DRAG_INFO_2;
+
+        gtk_drag_dest_set(plvbox,
+                          GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT |
+                          GTK_DEST_DEFAULT_DROP, target_entry, i, GDK_ACTION_LINK);
+
+        //Connect the signal for DnD
+        g_signal_connect(GTK_OBJECT(plvbox), "drag_data_received",
+                         GTK_SIGNAL_FUNC(playlist_drop_callback), NULL);
+
 
         playlisttip = gtk_tooltips_new();
 
-		count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL);
+        count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL);
         renderer = gtk_cell_renderer_text_new();
-        column = gtk_tree_view_column_new_with_attributes(ngettext("Item to play","Items to Play",count),
-                                                          renderer,
-                                                          "text", DESCRIPTION_COLUMN, NULL);
+        column =
+            gtk_tree_view_column_new_with_attributes(ngettext
+                                                     ("Item to play", "Items to Play", count),
+                                                     renderer, "text", DESCRIPTION_COLUMN, NULL);
         if (playlistname != NULL) {
             coltitle = g_strdup_printf(_("%s items"), playlistname);
             gtk_tree_view_column_set_title(column, coltitle);
             g_free(coltitle);
         }
         gtk_tree_view_column_set_expand(column, TRUE);
-        gtk_tree_view_column_set_max_width(column, 40);
+        //gtk_tree_view_column_set_max_width(column, 40);
+        gtk_tree_view_column_set_resizable(column, TRUE);
         gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
 
         renderer = gtk_cell_renderer_text_new();
@@ -615,20 +778,23 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
                                                           renderer, "text", ARTIST_COLUMN, NULL);
         gtk_tree_view_column_set_expand(column, TRUE);
         gtk_tree_view_column_set_max_width(column, 20);
+        gtk_tree_view_column_set_resizable(column, TRUE);
         gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
 
 
         renderer = gtk_cell_renderer_text_new();
         column = gtk_tree_view_column_new_with_attributes(_("Length"),
                                                           renderer, "text", LENGTH_COLUMN, NULL);
-        gtk_tree_view_column_set_expand(column, FALSE);
+        //gtk_tree_view_column_set_expand(column, FALSE);
         gtk_tree_view_column_set_alignment(column, 1.0);
+        gtk_tree_view_column_set_resizable(column, FALSE);
         gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
 
 
         renderer = gtk_cell_renderer_text_new();
         column = gtk_tree_view_column_new_with_attributes("", renderer, "text", COUNT_COLUMN, NULL);
-        gtk_tree_view_column_set_expand(column, FALSE);
+        //gtk_tree_view_column_set_expand(column, FALSE);
+        gtk_tree_view_column_set_resizable(column, FALSE);
         gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
 
 
@@ -692,6 +858,24 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
         gtk_box_pack_start(GTK_BOX(ctrlbox), clear, FALSE, FALSE, 0);
         g_signal_connect(GTK_OBJECT(clear), "clicked", GTK_SIGNAL_FUNC(clear_playlist), list);
 
+        up = gtk_button_new();
+        tooltip = gtk_tooltips_new();
+        gtk_tooltips_set_tip(tooltip, up, _("Move Item Up"), NULL);
+        gtk_button_set_image(GTK_BUTTON(up),
+                             gtk_image_new_from_stock(GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU));
+        gtk_box_pack_start(GTK_BOX(ctrlbox), up, FALSE, FALSE, 0);
+        g_signal_connect(GTK_OBJECT(up), "clicked", GTK_SIGNAL_FUNC(move_item_up), list);
+        gtk_widget_set_sensitive(up, FALSE);
+
+        down = gtk_button_new();
+        tooltip = gtk_tooltips_new();
+        gtk_tooltips_set_tip(tooltip, down, _("Move Item Down"), NULL);
+        gtk_button_set_image(GTK_BUTTON(down),
+                             gtk_image_new_from_stock(GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_MENU));
+        gtk_box_pack_start(GTK_BOX(ctrlbox), down, FALSE, FALSE, 0);
+        g_signal_connect(GTK_OBJECT(down), "clicked", GTK_SIGNAL_FUNC(move_item_down), list);
+        gtk_widget_set_sensitive(down, FALSE);
+
 
         accel_group = gtk_accel_group_new();
         gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
@@ -703,7 +887,7 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
 
         scrolled = gtk_scrolled_window_new(NULL, NULL);
         gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
-                                       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+                                       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
         gtk_container_add(GTK_CONTAINER(scrolled), list);
 
         gtk_box_pack_start(GTK_BOX(plvbox), scrolled, TRUE, TRUE, 0);
@@ -714,22 +898,24 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
 
         gtk_paned_pack2(GTK_PANED(pane), plvbox, FALSE, FALSE);
 
-		if (vertical_layout) {
-        	gtk_widget_set_size_request(plvbox, -1, 150);
-			if (idledata->videopresent)
-				gtk_window_resize(GTK_WINDOW(window),stored_window_width,stored_window_height+150);
-		} else {	
-	        gtk_widget_set_size_request(plvbox, 300, -1);
-			if (idledata->videopresent)
-				gtk_window_resize(GTK_WINDOW(window),stored_window_width+300,stored_window_height);
-		}
+        if (vertical_layout) {
+            gtk_widget_set_size_request(plvbox, -1, 150);
+            if (idledata->videopresent)
+                gtk_window_resize(GTK_WINDOW(window), stored_window_width,
+                                  stored_window_height + 150);
+        } else {
+            gtk_widget_set_size_request(plvbox, 300, -1);
+            if (idledata->videopresent)
+                gtk_window_resize(GTK_WINDOW(window), stored_window_width + 300,
+                                  stored_window_height);
+        }
         gtk_widget_show_all(plvbox);
         if (idledata->videopresent == FALSE) {
             gtk_widget_show(GTK_WIDGET(fixed));
         } else {
-			gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, TRUE);
-		}
-		
+            gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, TRUE);
+        }
+
         playlist_popup_menu = GTK_MENU(gtk_menu_new());
         playlist_set_subtitle =
             GTK_MENU_ITEM(gtk_image_menu_item_new_with_mnemonic(_("_Set Subtitle")));
@@ -739,8 +925,7 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
         gtk_menu_append(playlist_popup_menu, GTK_WIDGET(playlist_set_subtitle));
         g_signal_connect_swapped(G_OBJECT(list),
                                  "button_press_event",
-                                 G_CALLBACK(playlist_popup_handler),
-                                 G_OBJECT(playlist_popup_menu));
+                                 G_CALLBACK(playlist_popup_handler), G_OBJECT(playlist_popup_menu));
         gtk_widget_show_all(GTK_WIDGET(playlist_popup_menu));
 
 
