@@ -31,7 +31,10 @@ GtkMenu *playlist_popup_menu;
 GtkMenuItem *playlist_set_subtitle;
 gint window_width, window_height;
 gint filecount;
+GtkWidget *up;
+GtkWidget *down;
 
 void update_gui();
 void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data);
 void add_item_to_playlist_callback(gpointer data, gpointer user_data);
+void add_folder_to_playlist_callback(gpointer data, gpointer user_data);
