@@ -23,6 +23,9 @@
  */
 
 #include "support.h"
+static char *device = "default";
+static char *pcm_mix = "PCM";
+static char *master_mix = "Master";
 
 void strip_unicode(gchar * data, gsize len)
 {
@@ -48,7 +51,7 @@ gint detect_playlist(gchar * filename)
     gchar *file;
     GtkTreeViewColumn *column;
     gchar *coltitle;
-	gint count;
+    gint count;
 
     if (g_strncasecmp(filename, "cdda://", 7) == 0) {
         playlist = 1;
@@ -122,8 +125,8 @@ gint detect_playlist(gchar * filename)
         playlistname = NULL;
         if (GTK_WIDGET(list)) {
             column = gtk_tree_view_get_column(GTK_TREE_VIEW(list), 0);
-			count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL);
-            coltitle = g_strdup_printf(ngettext("Item to Play","Items to Play",count));
+            count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL);
+            coltitle = g_strdup_printf(ngettext("Item to Play", "Items to Play", count));
             gtk_tree_view_column_set_title(column, coltitle);
             g_free(coltitle);
         }
@@ -137,7 +140,7 @@ gint parse_playlist(gchar * filename)
     gint ret = 0;
     GtkTreeViewColumn *column;
     gchar *coltitle;
-	gint count;
+    gint count;
 
     // try and parse a playlist in various forms
     // if a parsing does not work then, return 0
@@ -170,8 +173,8 @@ gint parse_playlist(gchar * filename)
         playlistname = NULL;
         if (GTK_WIDGET(list)) {
             column = gtk_tree_view_get_column(GTK_TREE_VIEW(list), 0);
-			count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL);
-            coltitle = g_strdup_printf(ngettext("Item to Play","Items to Play",count));
+            count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL);
+            coltitle = g_strdup_printf(ngettext("Item to Play", "Items to Play", count));
             gtk_tree_view_column_set_title(column, coltitle);
             g_free(coltitle);
         }
@@ -325,7 +328,7 @@ gint parse_cdda(gchar * filename)
     gchar *stdout = NULL;
     gchar *stderr = NULL;
     gchar *av[255];
-    gint ac = 0;
+    gint ac = 0, i;
     gint ret = 0;
     gchar **output;
     gchar *track = NULL;
@@ -336,6 +339,7 @@ gint parse_cdda(gchar * filename)
     gint num;
     GtkTreeIter localiter;
     gboolean ok;
+    gint addcount = 0;
 
     if (g_strncasecmp(filename, "cdda://", 7) != 0) {
         return 0;
@@ -356,9 +360,16 @@ gint parse_cdda(gchar * filename)
 
         g_spawn_sync(NULL, av, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &stdout, &stderr,
                      &exit_status, &error);
+        for (i = 0; i < ac; i++) {
+            g_free(av[i]);
+        }
         if (error != NULL) {
             printf("Error when running: %s\n", error->message);
             g_error_free(error);
+            if (stdout != NULL)
+                g_free(stdout);
+            if (stderr != NULL)
+                g_free(stderr);
             error = NULL;
         }
         output = g_strsplit(stdout, "\n", 0);
@@ -419,6 +430,7 @@ gint parse_cdda(gchar * filename)
                                        PLAYLIST_COLUMN, 0,
                                        ARTIST_COLUMN, artist,
                                        SUBTITLE_COLUMN, NULL, LENGTH_COLUMN, length, -1);
+                    addcount++;
                 }
                 if (track != NULL) {
                     g_free(track);
@@ -435,6 +447,21 @@ gint parse_cdda(gchar * filename)
             }
             ac++;
         }
+
+        // printf("add count = %i \n",addcount);
+        if (addcount == 0) {
+            ac = 0;
+            while (output[ac] != NULL) {
+                if (g_strncasecmp(output[ac], "ID_CDDA_TRACK__", strlen("ID_CDDA_TRACK_")) == 0) {
+                    sscanf(output[ac], "ID_CDDA_TRACK_%i", &num);
+                    track = g_strdup_printf("cdda://%i", num);
+                    add_item_to_playlist(track, 0);
+                    g_free(track);
+                }
+                ac++;
+            }
+        }
+
         g_strfreev(output);
         if (stdout != NULL)
             g_free(stdout);
@@ -462,11 +489,13 @@ gint parse_dvd(gchar * filename)
     gchar *stdout;
     gchar *stderr;
     gchar *av[255];
-    gint ac = 0;
+    gint ac = 0, i;
     gint ret = 0;
     gchar **output;
     gchar *track;
     gint num;
+    gchar *error_msg = NULL;
+    GtkWidget *dialog;
 
     if (g_strncasecmp(filename, "dvd://", strlen("dvd://")) == 0) {     // || g_strncasecmp(filename,"dvdnav://",strlen("dvdnav://")) == 0) {
         playlist = 0;
@@ -479,6 +508,10 @@ gint parse_dvd(gchar * filename)
         av[ac++] = g_strdup_printf("-frames");
         av[ac++] = g_strdup_printf("0");
         av[ac++] = g_strdup_printf("-identify");
+        if (idledata->device != NULL) {
+            av[ac++] = g_strdup_printf("-dvd-device");
+            av[ac++] = g_strdup_printf("%s", idledata->device);
+        }
         av[ac++] = g_strdup_printf("dvd://");
         av[ac] = NULL;
 
@@ -486,12 +519,24 @@ gint parse_dvd(gchar * filename)
 
         g_spawn_sync(NULL, av, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &stdout, &stderr,
                      &exit_status, &error);
+        for (i = 0; i < ac; i++) {
+            g_free(av[i]);
+        }
         if (error != NULL) {
             printf("Error when running: %s\n", error->message);
             g_error_free(error);
+            if (stdout != NULL)
+                g_free(stdout);
+            if (stderr != NULL)
+                g_free(stderr);
             error = NULL;
         }
         output = g_strsplit(stdout, "\n", 0);
+        if (strstr(stderr, "Couldn't open DVD device:") != NULL) {
+            error_msg =
+                g_strdup_printf(_("Couldn't open DVD device: %s"),
+                                stderr + strlen("Couldn't open DVD device: "));
+        }
         ac = 0;
         while (output[ac] != NULL) {
             if (g_strncasecmp(output[ac], "ID_DVD_TITLE_", strlen("ID_DVD_TITLE_")) == 0) {
@@ -503,10 +548,20 @@ gint parse_dvd(gchar * filename)
                 }
             }
             ac++;
+
         }
         g_strfreev(output);
-
-        ret = 1;
+        if (error_msg != NULL) {
+            dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
+                                            GTK_BUTTONS_CLOSE, error_msg);
+            gtk_window_set_title(GTK_WINDOW(dialog), _("GNOME MPlayer Error"));
+            gtk_dialog_run(GTK_DIALOG(dialog));
+            gtk_widget_destroy(dialog);
+            g_free(error_msg);
+            ret = 0;
+        } else {
+            ret = 1;
+        }
     } else {
         return 0;
     }
@@ -598,6 +653,9 @@ gboolean update_mplayer_config()
         g_key_file_remove_key(config, "gnome-mplayer", "slang", NULL);
     }
 
+    g_key_file_remove_key(config, "gnome-mplayer", "really-quiet", NULL);
+    g_key_file_set_string(config, "gnome-mplayer", "msglevel", "5");
+
     data = g_key_file_to_data(config, NULL, NULL);
     //printf("%i\n%s", strlen(data), data);
 
@@ -668,6 +726,8 @@ gboolean streaming_media(gchar * filename)
 
     ret = TRUE;
 
+    if (filename == NULL)
+        return FALSE;
     if (strstr(filename, "dvd://") != NULL) {
         ret = FALSE;
     } else if (strstr(filename, "dvdnav://") != NULL) {
@@ -706,6 +766,21 @@ gboolean device_name(gchar * filename)
     return ret;
 }
 
+gchar *metadata_to_utf8(gchar * string)
+{
+    const gchar *lang;
+    lang = g_getenv("LANG");
+
+    if (metadata_codepage != NULL && strlen(metadata_codepage) > 1) {
+        // zh_TW usually use BIG5 on tags, if the file is from Windows
+        if (g_utf8_validate(string, strlen(string), NULL) == FALSE) {
+            return g_convert(string, -1, "UTF-8", metadata_codepage, NULL, NULL, NULL);
+        }
+    }
+
+    return g_locale_to_utf8(string, -1, NULL, NULL, NULL);
+}
+
 void get_metadata(gchar * name, gchar ** title, gchar ** artist, gchar ** length)
 {
 
@@ -714,7 +789,7 @@ void get_metadata(gchar * name, gchar ** title, gchar ** artist, gchar ** length
     gchar *stdout = NULL;
     gchar *stderr = NULL;
     gchar *av[255];
-    gint ac = 0;
+    gint ac = 0, i;
     gchar **output;
     gfloat seconds;
     gint hour = 0;
@@ -737,10 +812,20 @@ void get_metadata(gchar * name, gchar ** title, gchar ** artist, gchar ** length
 
     g_spawn_sync(NULL, av, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &stdout, &stderr,
                  &exit_status, &error);
+
+    for (i = 0; i < ac; i++) {
+        g_free(av[i]);
+    }
+
     if (error != NULL) {
         printf("Error when running: %s\n", error->message);
         g_error_free(error);
         error = NULL;
+        if (stdout != NULL)
+            g_free(stdout);
+        if (stderr != NULL)
+            g_free(stderr);
+        return;
     }
     output = g_strsplit(stdout, "\n", 0);
     ac = 0;
@@ -768,7 +853,7 @@ void get_metadata(gchar * name, gchar ** title, gchar ** artist, gchar ** length
         if (g_strncasecmp(output[ac], "ID_CLIP_INFO_NAME", strlen("ID_CLIP_INFO_NAME")) == 0) {
             if (strstr(output[ac], "Title") != NULL || strstr(output[ac], "name") != NULL) {
                 localtitle = strstr(output[ac + 1], "=") + 1;
-                *title = g_locale_to_utf8(localtitle, -1, NULL, NULL, NULL);
+                *title = metadata_to_utf8(localtitle);
                 if (*title == NULL) {
                     *title = g_strdup(localtitle);
                     strip_unicode(*title, strlen(*title));
@@ -776,7 +861,7 @@ void get_metadata(gchar * name, gchar ** title, gchar ** artist, gchar ** length
             }
             if (strstr(output[ac], "Artist") != NULL || strstr(output[ac], "author") != NULL) {
                 localtitle = strstr(output[ac + 1], "=") + 1;
-                *artist = g_locale_to_utf8(localtitle, -1, NULL, NULL, NULL);
+                *artist = metadata_to_utf8(localtitle);
                 if (*artist == NULL) {
                     *artist = g_strdup(localtitle);
                     strip_unicode(*artist, strlen(*artist));
@@ -787,7 +872,7 @@ void get_metadata(gchar * name, gchar ** title, gchar ** artist, gchar ** length
         if (strstr(output[ac], "DVD Title:") != NULL
             && g_strncasecmp(name, "dvdnav://", strlen("dvdnav://")) == 0) {
             localtitle = g_strrstr(output[ac], ":") + 1;
-            *title = g_locale_to_utf8(localtitle, -1, NULL, NULL, NULL);
+            *title = metadata_to_utf8(localtitle);
             if (*title == NULL) {
                 *title = g_strdup(localtitle);
                 strip_unicode(*title, strlen(*title));
@@ -817,19 +902,20 @@ gint get_bitrate(gchar * name)
     gchar *stdout = NULL;
     gchar *stderr = NULL;
     gchar *av[255];
-    gint ac = 0;
+    gint ac = 0, i;
     gchar **output;
-	gint bitrate = 0;
-	gint vbitrate = -1;
-	gint abitrate = -1;
-	gchar *buf;
-	
-	if (name == NULL)
-		return 0;
-	
-	if (!g_file_test(name, G_FILE_TEST_EXISTS))
-		return 0;
-	
+    gint bitrate = 0;
+    gint vbitrate = -1;
+    gint abitrate = -1;
+    gchar *buf;
+    gint startsec = 0, endpos = 0;
+
+    if (name == NULL)
+        return 0;
+
+    if (!g_file_test(name, G_FILE_TEST_EXISTS))
+        return 0;
+
     av[ac++] = g_strdup_printf("mplayer");
     av[ac++] = g_strdup_printf("-vo");
     av[ac++] = g_strdup_printf("null");
@@ -846,23 +932,94 @@ gint get_bitrate(gchar * name)
 
     g_spawn_sync(NULL, av, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &stdout, &stderr,
                  &exit_status, &error);
+    for (i = 0; i < ac; i++) {
+        g_free(av[i]);
+    }
     if (error != NULL) {
         printf("Error when running: %s\n", error->message);
         g_error_free(error);
+        if (stdout != NULL)
+            g_free(stdout);
+        if (stderr != NULL)
+            g_free(stderr);
         error = NULL;
+        return 0;
     }
     output = g_strsplit(stdout, "\n", 0);
     ac = 0;
     while (output[ac] != NULL) {
-	    if (strstr(output[ac], "ID_VIDEO_BITRATE") != 0) {
-	        buf = strstr(output[ac], "ID_VIDEO_BITRATE") + strlen("ID_VIDEO_BITRATE=");
-	        vbitrate = g_strtod(buf, NULL);
-	    }
-	    if (strstr(output[ac], "ID_AUDIO_BITRATE") != 0) {
-	        buf = strstr(output[ac], "ID_AUDIO_BITRATE") + strlen("ID_AUDIO_BITRATE=");
-	        abitrate = g_strtod(buf, NULL);
-	    }		
+        // find the length      
+        if (strstr(output[ac], "ID_LENGTH") != 0) {
+            buf = strstr(output[ac], "ID_LENGTH") + strlen("ID_LENGTH=");
+            endpos = (gint) g_strtod(buf, NULL) / 6;    // pick a spot early in the file
+            startsec = endpos - 1;
+        }
         ac++;
+
+    }
+    g_strfreev(output);
+    if (stdout != NULL) {
+        g_free(stdout);
+        stdout = NULL;
+    }
+
+    if (stderr != NULL) {
+        g_free(stderr);
+        stderr = NULL;
+    }
+    if (verbose)
+        printf("ss=%i, ep = %i\n", startsec, endpos);
+
+    ac = 0;
+    av[ac++] = g_strdup_printf("mencoder");
+    av[ac++] = g_strdup_printf("-ovc");
+    av[ac++] = g_strdup_printf("copy");
+    av[ac++] = g_strdup_printf("-oac");
+    av[ac++] = g_strdup_printf("copy");
+    av[ac++] = g_strdup_printf("-o");
+    av[ac++] = g_strdup_printf("/dev/null");
+    av[ac++] = g_strdup_printf("-quiet");
+    av[ac++] = g_strdup_printf("-ss");
+    av[ac++] = g_strdup_printf("%i", startsec);
+    av[ac++] = g_strdup_printf("-endpos");
+    av[ac++] = g_strdup_printf("%i", endpos);
+    av[ac++] = g_strdup_printf("%s", name);
+    av[ac] = NULL;
+
+    error = NULL;
+
+    g_spawn_sync(NULL, av, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &stdout, &stderr,
+                 &exit_status, &error);
+    for (i = 0; i < ac; i++) {
+        g_free(av[i]);
+    }
+    if (error != NULL) {
+        printf("Error when running: %s\n", error->message);
+        g_error_free(error);
+        if (stdout != NULL)
+            g_free(stdout);
+        if (stderr != NULL)
+            g_free(stderr);
+        error = NULL;
+        return 0;
+    }
+    output = g_strsplit(stdout, "\n", 0);
+    ac = 0;
+    while (output[ac] != NULL) {
+        if (strstr(output[ac], "Video stream") != 0) {
+            buf = g_strrstr(output[ac], "(");
+            if (buf != NULL) {
+                vbitrate = (gint) g_strtod(buf + sizeof(gchar), NULL);
+            }
+        }
+        if (strstr(output[ac], "Audio stream") != 0) {
+            buf = g_strrstr(output[ac], "(");
+            if (buf != NULL) {
+                abitrate = (gint) g_strtod(buf + sizeof(gchar), NULL);
+            }
+        }
+        ac++;
+
     }
 
     g_strfreev(output);
@@ -871,35 +1028,36 @@ gint get_bitrate(gchar * name)
     if (stderr != NULL)
         g_free(stderr);
 
-	if (vbitrate <= 0 && abitrate <= 0) {
-		bitrate = 0;
-	} else if (vbitrate == 0 && abitrate > 0) {
-		// mplayer knows there is video, but doesn't know the bit rate
-		// so we make the total bitrate 10 times the audio bitrate
-		bitrate = abitrate * 10;	
-	} else {
-		bitrate = vbitrate + abitrate;
-	}
-	
-	if (verbose)
-		printf ("bitrate = %i\n",bitrate);
-	return bitrate;
+    if (vbitrate <= 0 && abitrate <= 0) {
+        bitrate = 0;
+    } else if (vbitrate <= 0 && abitrate > 0) {
+        // mplayer knows there is video, but doesn't know the bit rate
+        // so we make the total bitrate 10 times the audio bitrate
+        bitrate = abitrate * 10;
+    } else {
+        bitrate = vbitrate + abitrate;
+    }
+
+    if (verbose)
+        printf("bitrate = %i\n", bitrate);
+    return bitrate;
 }
 
 
 GtkTreeIter add_item_to_playlist(gchar * itemname, gint playlist)
 {
     gchar *desc = NULL;
-    gchar *url = NULL;
     GtkTreeIter localiter;
     gchar *artist = NULL;
     gchar *length = NULL;
-	gchar *file = NULL;
+    gchar *file = NULL;
 
     if (!device_name(itemname) && !streaming_media(itemname)) {
         get_metadata(itemname, &desc, &artist, &length);
-		
+
         if (desc == NULL || (desc != NULL && strlen(desc) == 0)) {
+            if (desc != NULL)
+                g_free(desc);
             if (g_strrstr(itemname, "/") != NULL) {
                 desc = g_strdup_printf("%s", g_strrstr(itemname, "/") + sizeof(gchar));
             } else {
@@ -907,27 +1065,30 @@ GtkTreeIter add_item_to_playlist(gchar * itemname, gint playlist)
             }
         }
     } else {
-        url = g_strrstr(itemname, "http://");
 
         if (g_ascii_strncasecmp(itemname, "cdda://", strlen("cdda://")) == 0) {
             desc = g_strdup_printf("CD Track %s", itemname + strlen("cdda://"));
         } else if (g_ascii_strncasecmp(itemname, "file://", strlen("file://")) == 0) {
             file = itemname + sizeof(gchar) * strlen("file://");
-			get_metadata(file, &desc, &artist, &length);
-			if (desc == NULL || (desc != NULL && strlen(desc) == 0)) {
-	            if (g_strrstr(file, "/") != NULL) {
-	                desc = g_strdup_printf("%s", g_strrstr(file, "/") + sizeof(gchar));
-	            } else {
-	                desc = g_strdup(file);
-	            }
-	        }
+            get_metadata(file, &desc, &artist, &length);
+            if (desc == NULL || (desc != NULL && strlen(desc) == 0)) {
+                if (desc != NULL)
+                    g_free(desc);
+                if (g_strrstr(file, "/") != NULL) {
+                    desc = g_strdup_printf("%s", g_strrstr(file, "/") + sizeof(gchar));
+                } else {
+                    desc = g_strdup(file);
+                }
+            }
         } else if (device_name(itemname)) {
             if (g_strncasecmp(itemname, "dvdnav://", strlen("dvdnav://") == 0)) {
                 loop = 1;
             }
             get_metadata(itemname, &desc, &artist, &length);
-            if (desc == NULL|| (desc != NULL && strlen(desc) == 0))
-                desc = g_strdup_printf("Device - %s", itemname);
+            if (desc == NULL || (desc != NULL && strlen(desc) == 0))
+                if (desc != NULL)
+                    g_free(desc);
+            desc = g_strdup_printf("Device - %s", itemname);
 
         } else {
             desc = g_strdup_printf("Stream from %s", itemname);
@@ -953,7 +1114,15 @@ GtkTreeIter add_item_to_playlist(gchar * itemname, gint playlist)
                            SUBTITLE_COLUMN, subtitle, LENGTH_COLUMN, length, -1);
 
     }
-    g_free(desc);
+    if (desc != NULL)
+        g_free(desc);
+    if (artist != NULL)
+        g_free(artist);
+    if (subtitle != NULL)
+        g_free(subtitle);
+    if (length != NULL)
+        g_free(length);
+
     return localiter;
 
 }
@@ -1077,10 +1246,10 @@ void copy_playlist(GtkListStore * source, GtkListStore * dest)
     gchar *subtitle;
     gchar *length;
 
-	if (gtk_list_store_iter_is_valid(playliststore, &iter)) {
-	    gtk_tree_model_get(GTK_TREE_MODEL(dest), &iter, ITEM_COLUMN, &iterfilename, -1);
-	}
-	
+    if (gtk_list_store_iter_is_valid(playliststore, &iter)) {
+        gtk_tree_model_get(GTK_TREE_MODEL(dest), &iter, ITEM_COLUMN, &iterfilename, -1);
+    }
+
     gtk_list_store_clear(dest);
     if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(source), &sourceiter)) {
         do {
@@ -1112,18 +1281,18 @@ void copy_playlist(GtkListStore * source, GtkListStore * dest)
     }
 
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(dest), &a);
-	if (gtk_list_store_iter_is_valid(playliststore, &a)) {
-	    do {
-	        gtk_tree_model_get(GTK_TREE_MODEL(dest), &a, ITEM_COLUMN, &localfilename, -1);
-	        // printf("iter = %s   local = %s \n",iterfilename,localfilename);
-	        if (g_ascii_strcasecmp(iterfilename, localfilename) == 0) {
-	            // we found the current iter
-	            break;
-	        }
-	        g_free(localfilename);
-	    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(dest), &a));
-    	iter = a;
-	}
+    if (gtk_list_store_iter_is_valid(playliststore, &a)) {
+        do {
+            gtk_tree_model_get(GTK_TREE_MODEL(dest), &a, ITEM_COLUMN, &localfilename, -1);
+            // printf("iter = %s   local = %s \n",iterfilename,localfilename);
+            if (g_ascii_strcasecmp(iterfilename, localfilename) == 0) {
+                // we found the current iter
+                break;
+            }
+            g_free(localfilename);
+        } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(dest), &a));
+        iter = a;
+    }
     g_free(iterfilename);
 }
 
@@ -1178,3 +1347,93 @@ void randomize_playlist(GtkListStore * store)
     }
     g_rand_free(rand);
 }
+
+gdouble get_alsa_volume()
+{
+    gdouble vol = 100.0;
+
+#ifdef HAVE_ASOUNDLIB
+
+    gint err;
+    snd_mixer_t *mhandle;
+    snd_mixer_elem_t *elem;
+    snd_mixer_selem_id_t *sid;
+    glong get_vol, pmin, pmax;
+    gfloat f_multi;
+    gboolean found = FALSE;
+
+    if ((err = snd_mixer_open(&mhandle, 0)) < 0) {
+        if (verbose)
+            printf("Mixer open error %s\n", snd_strerror(err));
+        return vol;
+    }
+
+    if ((err = snd_mixer_attach(mhandle, device)) < 0) {
+        if (verbose)
+            printf("Mixer attach error %s\n", snd_strerror(err));
+        return vol;
+    }
+
+    if ((err = snd_mixer_selem_register(mhandle, NULL, NULL)) < 0) {
+        if (verbose)
+            printf("Mixer register error %s\n", snd_strerror(err));
+        return vol;
+    }
+
+    if ((err = snd_mixer_load(mhandle)) < 0) {
+        if (verbose)
+            printf("Mixer load error %s\n", snd_strerror(err));
+        return vol;
+    }
+
+
+    snd_mixer_selem_id_alloca(&sid);
+    snd_mixer_selem_id_set_index(sid, 0);
+    snd_mixer_selem_id_set_name(sid, pcm_mix);
+
+    elem = snd_mixer_find_selem(mhandle, sid);
+    if (!elem) {
+        if (verbose)
+            printf("Unable to find PCM Mixer control, trying Master\n");
+    } else {
+        snd_mixer_selem_get_playback_volume_range(elem, &pmin, &pmax);
+        f_multi = (100 / (float) (pmax - pmin));
+        snd_mixer_selem_get_playback_volume(elem, 0, &get_vol);
+        vol = (gdouble) ((get_vol - pmin) * f_multi);
+        if (verbose) {
+            printf("PCM Range is %li to %li \n", pmin, pmax);
+            printf("PCM Volume is %lf\n", vol);
+        }
+        found = TRUE;
+    }
+
+    if (!found) {
+        snd_mixer_selem_id_alloca(&sid);
+        snd_mixer_selem_id_set_index(sid, 0);
+        snd_mixer_selem_id_set_name(sid, master_mix);
+
+        elem = snd_mixer_find_selem(mhandle, sid);
+        if (!elem) {
+            if (verbose)
+                printf("Unable to find Master Mixer control, using default of 100%%\n");
+        } else {
+            snd_mixer_selem_get_playback_volume_range(elem, &pmin, &pmax);
+            f_multi = (100 / (float) (pmax - pmin));
+            snd_mixer_selem_get_playback_volume(elem, 0, &get_vol);
+            vol = (gdouble) ((get_vol - pmin) * f_multi);
+            if (verbose) {
+                printf("Master Range is %li to %li \n", pmin, pmax);
+                printf("Master Volume is %lf\n", vol);
+            }
+            found = TRUE;
+        }
+    }
+
+    snd_mixer_close(mhandle);
+
+#endif
+
+    if (verbose)
+        printf("Using volume of %3.2lf\n", vol);
+    return vol;
+}
