@@ -31,10 +31,15 @@
 #include <stdlib.h>
 #include <unistd.h>
 
+#ifdef HAVE_ASOUNDLIB
+#include <asoundlib.h>
+#endif
+
 gint get_bitrate(gchar * name);
 void strip_unicode(gchar * data, gsize len);
 gint play_file(gchar * filename, gint playlist);
 gint detect_playlist(gchar * filename);
+gchar *metadata_to_utf8(gchar * string);
 gint parse_playlist(gchar * filename);
 gint parse_basic(gchar * filename);
 gint parse_ram(gchar * filename);
@@ -51,4 +56,5 @@ GtkTreeIter add_item_to_playlist(gchar * itemname, gint playlist);
 gboolean next_item_in_playlist(GtkTreeIter * iter);
 void copy_playlist(GtkListStore * source, GtkListStore * dest);
 void randomize_playlist(GtkListStore * store);
+gdouble get_alsa_volume();
 #endif                          // _SUPPORT_H
