@@ -41,7 +41,8 @@ gboolean send_command(gchar * command)
 {
     gint ret;
 
-    //printf("send command = %s\n",command);
+    if (verbose > 1)
+        printf("send command = %s\n", command);
     ret = write(std_in, command, strlen(command));
     fsync(std_in);
     if (ret < 0) {
@@ -60,17 +61,17 @@ gboolean play(void *data)
 
     if (ok_to_play && p != NULL) {
         play_file(p->filename, p->playlist);
-		if (gtk_list_store_iter_is_valid(playliststore, &iter)) {		
-	        gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, COUNT_COLUMN, &count, -1);
-	        gtk_list_store_set(playliststore, &iter, COUNT_COLUMN, count + 1, -1);
-	        if (GTK_IS_TREE_SELECTION(selection)) {
-	            path = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &iter);
-	            gtk_tree_selection_select_path(selection, path);
-				if (GTK_IS_WIDGET(list))
-					gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list),path,NULL,FALSE,0,0);
-	            gtk_tree_path_free(path);
-	        }
-		}
+        if (gtk_list_store_iter_is_valid(playliststore, &iter)) {
+            gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, COUNT_COLUMN, &count, -1);
+            gtk_list_store_set(playliststore, &iter, COUNT_COLUMN, count + 1, -1);
+            if (GTK_IS_TREE_SELECTION(selection)) {
+                path = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &iter);
+                gtk_tree_selection_select_path(selection, path);
+                if (GTK_IS_WIDGET(list))
+                    gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list), path, NULL, FALSE, 0, 0);
+                gtk_tree_path_free(path);
+            }
+        }
     }
     g_free(p);
 
@@ -102,6 +103,8 @@ gboolean thread_reader_error(GIOChannel * source, GIOCondition condition, gpoint
 
 
     if (state == QUIT) {
+        if (verbose > 1)
+            printf("Thread Error: state = QUIT, shutting down\n");
         g_idle_add(set_stop, idledata);
         state = QUIT;
         g_source_remove(watch_in_id);
@@ -120,14 +123,33 @@ gboolean thread_reader_error(GIOChannel * source, GIOCondition condition, gpoint
         error_msg = g_strdup(mplayer_output->str);
     }
 
+    if (strstr(mplayer_output->str, "signal") != NULL) {
+        error_msg = g_strdup(mplayer_output->str);
+    }
+
     if (strstr(mplayer_output->str, "Failed to open") != NULL) {
         if (strstr(mplayer_output->str, "LIRC") == NULL &&
             strstr(mplayer_output->str, "/dev/rtc") == NULL &&
             strstr(mplayer_output->str, "registry file") == NULL) {
-            error_msg = g_strdup(mplayer_output->str);
+            // error_msg = g_strdup(mplayer_output->str);
+            error_msg =
+                g_strdup_printf(_("Failed to open %s"),
+                                mplayer_output->str + strlen("Failed to open "));
         }
     }
 
+    if (strstr(mplayer_output->str, "Couldn't open DVD device:") != NULL) {
+        error_msg =
+            g_strdup_printf(_("Couldn't open DVD device: %s"),
+                            mplayer_output->str + strlen("Couldn't open DVD device: "));
+    }
+
+    if (strstr(mplayer_output->str, "Cannot load subtitles: ") != NULL) {
+        error_msg =
+            g_strdup_printf(_("Cannot load subtitles: %s"),
+                            mplayer_output->str + strlen("Cannot load subtitles: "));
+    }
+
     if (strstr(mplayer_output->str, "Failed to initiate \"video/X-ASF-PF\" RTP subsession") != NULL) {
         dontplaynext = TRUE;
         playback_error = ERROR_RETRY_WITH_PLAYLIST;
@@ -144,7 +166,7 @@ gboolean thread_reader_error(GIOChannel * source, GIOCondition condition, gpoint
     if (error_msg != NULL) {
         dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
                                         GTK_BUTTONS_CLOSE, error_msg);
-        gtk_window_set_title(GTK_WINDOW(dialog), "GNOME MPlayer Error");
+        gtk_window_set_title(GTK_WINDOW(dialog), _("GNOME MPlayer Error"));
         gtk_dialog_run(GTK_DIALOG(dialog));
         gtk_widget_destroy(dialog);
         g_free(error_msg);
@@ -172,6 +194,7 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
     gchar *cdartist;
     gchar *utf8name;
     gchar *utf8artist;
+    gdouble old_pos;
 
     if (source == NULL) {
         g_source_remove(watch_err_id);
@@ -221,6 +244,13 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         }
     }
 
+    if (strstr(mplayer_output->str, "ID_SIGNAL") != NULL) {
+        g_mutex_unlock(thread_running);
+        g_string_free(mplayer_output, TRUE);
+        state = QUIT;
+        return FALSE;
+    }
+
     if ((strstr(mplayer_output->str, "A:") != NULL) || (strstr(mplayer_output->str, "V:") != NULL)) {
         g_string_free(mplayer_output, TRUE);
         return TRUE;
@@ -288,9 +318,20 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
     }
 
     if (strstr(mplayer_output->str, "ANS_TIME_POSITION") != 0) {
+        old_pos = idledata->position;
         buf = strstr(mplayer_output->str, "ANS_TIME_POSITION");
         sscanf(buf, "ANS_TIME_POSITION=%lf", &idledata->position);
+        if (idledata->position < old_pos) {
+            send_command("get_time_length\n");
+            state = PLAYING;
+        }
         g_idle_add(set_progress_time, idledata);
+        if ((int) idledata->length != 0) {
+            idledata->percent = idledata->position / idledata->length;
+            g_idle_add(set_progress_value, idledata);
+        } else {
+            send_command("get_percent_pos\n");
+        }
     }
 
     if (strstr(mplayer_output->str, "ANS_stream_pos") != 0) {
@@ -361,12 +402,12 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
                     //g_strlcpy(idledata->info, message, 1024);
                     //g_free(message);
                     //g_idle_add(set_media_info, idledata);
-                    utf8name = g_locale_to_utf8(parse[name], -1, NULL, NULL, NULL);
+                    utf8name = metadata_to_utf8(parse[name]);
                     if (utf8name == NULL) {
                         strip_unicode(parse[name], strlen(parse[name]));
                         utf8name = g_strdup(parse[name]);
                     }
-                    utf8artist = g_locale_to_utf8(parse[artist], -1, NULL, NULL, NULL);
+                    utf8artist = metadata_to_utf8(parse[artist]);
                     if (utf8artist == NULL) {
                         strip_unicode(parse[artist], strlen(parse[artist]));
                         utf8artist = g_strdup(parse[artist]);
@@ -450,6 +491,7 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         //printf("Buffer = %s\n",buf);
         buf = g_strdup_printf(_("Cache fill: %2.2f%%"), percent);
         g_strlcpy(idledata->progress_text, buf, 1024);
+        g_free(buf);
         g_idle_add(set_progress_text, idledata);
         idledata->percent = percent / 100.0;
         g_idle_add(set_progress_value, idledata);
@@ -458,6 +500,7 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
     if (strstr(mplayer_output->str, "Connecting") != 0) {
         buf = g_strdup_printf(_("Connecting"));
         g_strlcpy(idledata->progress_text, buf, 1024);
+        g_free(buf);
         g_idle_add(set_progress_text, idledata);
         idledata->percent = 0.0;
         g_idle_add(set_progress_value, idledata);
@@ -505,6 +548,12 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         g_strlcpy(idledata->audio_samplerate, buf, 16);
     }
 
+    if (strstr(mplayer_output->str, "ID_AUDIO_NCH") != 0) {
+        g_string_truncate(mplayer_output, mplayer_output->len - 1);
+        buf = strstr(mplayer_output->str, "ID_AUDIO_NCH") + strlen("ID_AUDIO_NCH=");
+        g_strlcpy(idledata->audio_channels, buf, 16);
+    }
+
     if (strstr(mplayer_output->str, "File not found") != 0) {
     }
 
@@ -512,6 +561,19 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         error_msg = g_strdup(mplayer_output->str);
     }
 
+    if (strstr(mplayer_output->str, "*** screenshot") != 0) {
+        buf = strstr(mplayer_output->str, "'") + 1;
+        buf[12] = '\0';
+        message = g_strdup_printf(_("Screenshot saved to '%s'"), buf);
+        dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO,
+                                        GTK_BUTTONS_OK, message);
+        gtk_window_set_title(GTK_WINDOW(dialog), _("GNOME MPlayer Notification"));
+        gtk_dialog_run(GTK_DIALOG(dialog));
+        gtk_widget_destroy(dialog);
+        g_free(message);
+        message = NULL;
+    }
+
     if (strstr(mplayer_output->str, "ICY Info") != NULL) {
         buf = strstr(mplayer_output->str, "'");
         if (message) {
@@ -527,11 +589,18 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
                 }
             }
         }
-        if (message)
+        if (message) {
             g_strlcpy(idledata->info, message, 1024);
+            g_free(message);
+            message = NULL;
+        }
         g_idle_add(set_media_info, idledata);
     }
 
+    if (verbose > 1) {
+        printf("MPLAYER OUTPUT: %s\n", mplayer_output->str);
+    }
+
     if (error_msg != NULL) {
         dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
                                         GTK_BUTTONS_OK, error_msg);
@@ -549,8 +618,8 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
 gboolean thread_query(gpointer data)
 {
     int size;
-	ThreadData *threaddata = (ThreadData *) data;
-	
+    ThreadData *threaddata = (ThreadData *) data;
+
     //printf("thread_query state = %i\n",state);
 
     // this function wakes up every 1/2 second and so 
@@ -568,25 +637,26 @@ gboolean thread_query(gpointer data)
         return FALSE;
     }
 
-	if (threaddata->done == TRUE) {
+    if (threaddata->done == TRUE) {
         if (verbose)
             printf("shutting down threadquery since threaddata->done is TRUE\n");
-	    g_free(threaddata);
-	    threaddata = NULL;
+        g_free(threaddata);
+        threaddata = NULL;
         return FALSE;
     }
 
     if (state == PLAYING) {
-        size = write(std_in, "get_percent_pos\n", strlen("get_percent_pos\n"));
+        // size = write(std_in, "get_percent_pos\n", strlen("get_percent_pos\n"));
+        size = write(std_in, "get_time_pos\n", strlen("get_time_pos\n"));
         if (size == -1) {
             shutdown();
             return FALSE;
         } else {
 
-            send_command("get_time_pos\n");
+            //send_command("get_time_pos\n");
             send_command("get_property stream_pos\n");
-			if (threaddata->streaming)
-				send_command("get_property metadata\n");
+            if (threaddata->streaming)
+                send_command("get_property metadata\n");
             g_idle_add(make_panel_and_mouse_invisible, NULL);
             return TRUE;
         }
@@ -608,11 +678,13 @@ gpointer launch_player(gpointer data)
     gint arg = 0;
     gchar *filename;
     gint count;
-    PlayData *p = (PlayData *) g_malloc(sizeof(PlayData));;
+    PlayData *p = (PlayData *) g_malloc(sizeof(PlayData));
+    gchar *fontname;
+    gchar *size;
 
     ThreadData *threaddata = (ThreadData *) data;
 
-    videopresent = 1;
+    videopresent = 0;
     playback_error = NO_ERROR;
 
     while (embed_window != -1 && !GTK_WIDGET_VISIBLE(window)) {
@@ -628,7 +700,7 @@ gpointer launch_player(gpointer data)
     idledata->width = 1;
     idledata->height = 1;
     idledata->videopresent = 1;
-    idledata->volume = 100.0;
+    idledata->volume = volume;
     idledata->length = 0.0;
 
     g_idle_add(set_progress_value, idledata);
@@ -636,12 +708,17 @@ gpointer launch_player(gpointer data)
     g_idle_add(set_media_info, idledata);
     //g_idle_add(set_window_visible, idledata);
 
-    argv[arg++] = g_strdup_printf("mplayer");
-    if (vo != NULL) {
+    if (mplayer_bin == NULL || !g_file_test(mplayer_bin, G_FILE_TEST_EXISTS)) {
+        argv[arg++] = g_strdup_printf("mplayer");
+    } else {
+        argv[arg++] = g_strdup_printf("%s", mplayer_bin);
+    }
+    if (vo != NULL && strlen(vo) > 0) {
         argv[arg++] = g_strdup_printf("-profile");
         argv[arg++] = g_strdup_printf("gnome-mplayer");
     }
-    argv[arg++] = g_strdup_printf("-quiet");
+    if (verbose < 2)
+        argv[arg++] = g_strdup_printf("-quiet");
     argv[arg++] = g_strdup_printf("-slave");
     argv[arg++] = g_strdup_printf("-identify");
 
@@ -649,20 +726,28 @@ gpointer launch_player(gpointer data)
     if (softvol)
         argv[arg++] = g_strdup_printf("-softvol");
 
-    argv[arg++] = g_strdup_printf("-framedrop");
+    if (!disable_framedrop)
+        argv[arg++] = g_strdup_printf("-framedrop");
+
+    if (!disable_deinterlace) {
+        argv[arg++] = g_strdup_printf("-vf-pre");
+        argv[arg++] = g_strdup_printf("yadif,softskip,scale");
+    }
+
     argv[arg++] = g_strdup_printf("-noconsolecontrols");
     argv[arg++] = g_strdup_printf("-osdlevel");
     argv[arg++] = g_strdup_printf("%i", osdlevel);
     if (strcmp(threaddata->filename, "dvdnav://") == 0) {
         argv[arg++] = g_strdup_printf("-mouse-movements");
+        argv[arg++] = g_strdup_printf("-nocache");
     } else {
         if (g_strncasecmp(threaddata->filename, "dvd://", strlen("dvd://")) == 0) {
             // argv[arg++] = g_strdup_printf("-nocache");
         } else {
             argv[arg++] = g_strdup_printf("-nomouseinput");
             if (threaddata->streaming && forcecache == FALSE) {
-                argv[arg++] = g_strdup_printf("-user-agent");
-                argv[arg++] = g_strdup_printf("NSPlayer");
+                //argv[arg++] = g_strdup_printf("-user-agent");
+                //argv[arg++] = g_strdup_printf("NSPlayer");
             } else {
                 argv[arg++] = g_strdup_printf("-cache");
                 argv[arg++] = g_strdup_printf("%i", cache_size);
@@ -708,15 +793,99 @@ gpointer launch_player(gpointer data)
         argv[arg++] = g_strdup_printf("-sub");
         argv[arg++] = g_strdup_printf("%s", threaddata->subtitle);
     }
+    // subtitle stuff
+    if (!disable_ass) {
+        argv[arg++] = g_strdup_printf("-ass");
+
+        // Simply ommiting '-embeddedfonts' did not work
+        if (disable_embeddedfonts)
+            argv[arg++] = g_strdup_printf("-noembeddedfonts");
+        else
+            argv[arg++] = g_strdup_printf("-embeddedfonts");
+
+        argv[arg++] = g_strdup_printf("-ass-font-scale");
+        argv[arg++] = g_strdup_printf("%1.1f", subtitle_scale);
+        if (subtitlefont != NULL && strlen(subtitlefont) > 0) {
+            fontname = g_strdup(subtitlefont);
+            size = g_strrstr(fontname, " ");
+            size[0] = '\0';
+            argv[arg++] = g_strdup_printf("-ass-force-style");
+            argv[arg++] = g_strdup_printf("FontName=%s", fontname);
+            g_free(fontname);
+        }
+
+        if (subtitle_color != NULL && strlen(subtitle_color) > 0) {
+            argv[arg++] = g_strdup_printf("-ass-color");
+            argv[arg++] = g_strdup_printf("%s", subtitle_color);
+        }
+
+    } else {
+        argv[arg++] = g_strdup_printf("-subfont-text-scale");
+        argv[arg++] = g_strdup_printf("%d", (int) (subtitle_scale * 5));        // 5 is the default
+
+        if (subtitlefont != NULL && strlen(subtitlefont) > 0) {
+            fontname = g_strdup(subtitlefont);
+            size = g_strrstr(fontname, " ");
+            size[0] = '\0';
+            argv[arg++] = g_strdup_printf("-subfont");
+            argv[arg++] = g_strdup_printf("%s", fontname);
+            g_free(fontname);
+        }
+    }
+
+    if (subtitle_codepage != NULL && strlen(subtitle_codepage) > 0) {
+        argv[arg++] = g_strdup_printf("-subcp");
+        argv[arg++] = g_strdup_printf("%s", subtitle_codepage);
+    }
+
+    if (pplevel > 0) {
+        argv[arg++] = g_strdup_printf("-vf-add");
+        argv[arg++] = g_strdup_printf("pp=ac/tn:a");
+        argv[arg++] = g_strdup_printf("-autoq");
+        argv[arg++] = g_strdup_printf("%d", pplevel);
+    }
+
+    if (extraopts != NULL) {
+        char **opts = g_strsplit(extraopts, " ", -1);
+        int i;
+        for (i = 0; opts[i] != NULL; i++)
+            argv[arg++] = g_strdup(opts[i]);
+        g_strfreev(opts);
+    }
+
+    argv[arg++] = g_strdup_printf("-vf-add");
+    argv[arg++] = g_strdup_printf("screenshot");
+
+    if (idledata->device != NULL) {
+        argv[arg++] = g_strdup_printf("-dvd-device");
+        argv[arg++] = g_strdup_printf("%s", idledata->device);
+    }
 
     if (playlist || threaddata->playlist)
         argv[arg++] = g_strdup_printf("-playlist");
     argv[arg] = g_strdup_printf("%s", threaddata->filename);
     argv[arg + 1] = NULL;
+
+    if (verbose > 1) {
+        arg = 0;
+        while (argv[arg] != NULL) {
+            printf("%s ", argv[arg++]);
+        }
+        printf("\n");
+    }
+
+    state = PAUSED;
     ok = g_spawn_async_with_pipes(NULL, argv, NULL,
                                   G_SPAWN_SEARCH_PATH,
                                   NULL, NULL, NULL, &std_in, &std_out, &std_err, NULL);
 
+    arg = 0;
+    while (argv[arg] != NULL) {
+        g_free(argv[arg]);
+        argv[arg] = NULL;
+        arg++;
+    }
+
     if (ok) {
         if (verbose)
             printf("Spawn succeeded for filename %s\n", threaddata->filename);
@@ -751,77 +920,51 @@ gpointer launch_player(gpointer data)
 
         g_idle_add(set_play, NULL);
 #ifdef GLIB2_14_ENABLED
-		g_timeout_add_seconds(1,thread_query, threaddata);
+        g_timeout_add_seconds(1, thread_query, threaddata);
 #else
         g_timeout_add(1000, thread_query, threaddata);
 #endif
-		
-    } else {
-        state = QUIT;
-        printf("Spawn failed for filename %s\n", threaddata->filename);
-    }
 
-    g_mutex_lock(thread_running);
-    if (verbose)
-        printf("Thread completing\n");
-	threaddata->done = TRUE;
-    g_source_remove(watch_in_id);
-    g_source_remove(watch_err_id);
-    g_source_remove(watch_in_hup_id);
-    idledata->percent = 1.0;
-    idledata->position = idledata->length;
-    g_idle_add(set_progress_value, idledata);
-    g_idle_add(set_progress_time, idledata);
+        g_mutex_lock(thread_running);
+        if (verbose)
+            printf("Thread completing\n");
+        threaddata->done = TRUE;
+        g_source_remove(watch_in_id);
+        g_source_remove(watch_err_id);
+        g_source_remove(watch_in_hup_id);
+        idledata->percent = 1.0;
+        idledata->position = idledata->length;
+        g_idle_add(set_progress_value, idledata);
+        g_idle_add(set_progress_time, idledata);
 
-    if (embed_window != 0 || control_id != 0) {
-        dbus_send_event("MediaComplete", 0);
-        dbus_open_next();
-    }
+        if (embed_window != 0 || control_id != 0) {
+            dbus_send_event("MediaComplete", 0);
+            dbus_open_next();
+        }
 
-    if (channel_in != NULL) {
-        g_io_channel_shutdown(channel_in, FALSE, NULL);
-		g_io_channel_unref(channel_in);
-        channel_in = NULL;
-    }
+        if (channel_in != NULL) {
+            g_io_channel_shutdown(channel_in, FALSE, NULL);
+            g_io_channel_unref(channel_in);
+            channel_in = NULL;
+        }
 
-    if (channel_out != NULL) {
-        g_io_channel_shutdown(channel_out, FALSE, NULL);
-		g_io_channel_unref(channel_out);
-        channel_out = NULL;
-    }
+        if (channel_out != NULL) {
+            g_io_channel_shutdown(channel_out, FALSE, NULL);
+            g_io_channel_unref(channel_out);
+            channel_out = NULL;
+        }
 
-    if (channel_err != NULL) {
-        g_io_channel_shutdown(channel_err, FALSE, NULL);
-		g_io_channel_unref(channel_err);
-        channel_err = NULL;
-    }
+        if (channel_err != NULL) {
+            g_io_channel_shutdown(channel_err, FALSE, NULL);
+            g_io_channel_unref(channel_err);
+            channel_err = NULL;
+        }
 
-    arg = 0;
-    while (argv[arg] != NULL) {
-        g_free(argv[arg]);
-        argv[arg] = NULL;
-        arg++;
-    }
+        dbus_enable_screensaver();
+        g_mutex_unlock(thread_running);
 
-	g_mutex_unlock(thread_running);
-    // printf("Thread done\n");
-
-    if (dontplaynext == FALSE) {
-        if (next_item_in_playlist(&iter)) {
-            if (gtk_list_store_iter_is_valid(playliststore, &iter)) {
-                gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN, &filename,
-                                   COUNT_COLUMN, &count, PLAYLIST_COLUMN, &playlist, -1);
-                g_strlcpy(idledata->info, filename, 4096);
-                g_idle_add(set_media_info, idledata);
-                g_strlcpy(p->filename, filename, 4096);
-                p->playlist = playlist;
-                g_idle_add(play, p);
-                g_free(filename);
-            }
-        } else {
-            // printf("end of thread playlist is empty\n");
-            if (loop) {
-                gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter);
+        if (dontplaynext == FALSE) {
+            if (next_item_in_playlist(&iter)) {
                 if (gtk_list_store_iter_is_valid(playliststore, &iter)) {
                     gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN, &filename,
                                        COUNT_COLUMN, &count, PLAYLIST_COLUMN, &playlist, -1);
@@ -832,17 +975,46 @@ gpointer launch_player(gpointer data)
                     g_idle_add(play, p);
                     g_free(filename);
                 }
+            } else {
+                // printf("end of thread playlist is empty\n");
+                if (loop) {
+                    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter);
+                    if (gtk_list_store_iter_is_valid(playliststore, &iter)) {
+                        gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN,
+                                           &filename, COUNT_COLUMN, &count, PLAYLIST_COLUMN,
+                                           &playlist, -1);
+                        g_strlcpy(idledata->info, filename, 4096);
+                        g_idle_add(set_media_info, idledata);
+                        g_strlcpy(p->filename, filename, 4096);
+                        p->playlist = playlist;
+                        g_idle_add(play, p);
+                        g_free(filename);
+                    }
+                }
+
+                if (quit_on_complete) {
+                    g_idle_add(set_quit, idledata);
+                }
             }
+        } else {
+            if (playback_error == ERROR_RETRY_WITH_PLAYLIST) {
+                g_strlcpy(p->filename, threaddata->filename, 4096);
+                p->playlist = 1;
+                g_idle_add(play, p);
+            }
+            dontplaynext = FALSE;
         }
+
     } else {
-        if (playback_error == ERROR_RETRY_WITH_PLAYLIST) {
-            g_strlcpy(p->filename, threaddata->filename, 4096);
-            p->playlist = 1;
-            g_idle_add(play, p);
-        }
-        dontplaynext = FALSE;
+        state = QUIT;
+        printf("Spawn failed for filename %s\n", threaddata->filename);
+        g_mutex_unlock(thread_running);
     }
 
+    // printf("Thread done\n");
+
+
+
     thread = NULL;
     return NULL;
 }
